/********************************************************************
 *
 * Copyright:
 *	1999-2000 MOTOROLA, INC. All Rights Reserved.  
 *  You are hereby granted a copyright license to use, modify, and
 *  distribute the SOFTWARE so long as this entire notice is
 *  retained without alteration in any modified and/or redistributed
 *  versions, and that such modified versions are clearly identified
 *  as such. No licenses are granted by implication, estoppel or
 *  otherwise under any patents or trademarks of Motorola, Inc. This 
 *  software is provided on an "AS IS" basis and without warranty.
 *
 *  To the maximum extent permitted by applicable law, MOTOROLA 
 *  DISCLAIMS ALL WARRANTIES WHETHER EXPRESS OR IMPLIED, INCLUDING 
 *  IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR
 *  PURPOSE AND ANY WARRANTY AGAINST INFRINGEMENT WITH REGARD TO THE 
 *  SOFTWARE (INCLUDING ANY MODIFIED VERSIONS THEREOF) AND ANY 
 *  ACCOMPANYING WRITTEN MATERIALS.
 * 
 *  To the maximum extent permitted by applicable law, IN NO EVENT
 *  SHALL MOTOROLA BE LIABLE FOR ANY DAMAGES WHATSOEVER (INCLUDING 
 *  WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS 
 *  INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY
 *  LOSS) ARISING OF THE USE OR INABILITY TO USE THE SOFTWARE.   
/*
 * File:		canc.c
 * Purpose:		MCF5272 CAN reference design software. Driver software for the 82C900 CAN controller.
 *
 */

#include "mcf5272.h"
#include "init.h"
#include "canvar.h"


void CAN_SetPageReg(uint8 PageNumber)
{
/* This routine sets the page register on the 82C900 with the upper 4 MSBs of the address of the register to be accessed. 
Called from QSPI_SendByte, QSPI_ReadByte*/

 MCF5272_IMM *imm = mcf5272_get_immp();

/*Set RAM address pointer to top of Tx RAM*/
 MCF5272_WR_QSPI_QAR(imm,MCF5272_QSPI_QAR_Tx);
/*Load page reg address into Tx RAM via the data reg. Indicate Wr.*/
 MCF5272_WR_QSPI_QDR(imm,CAN_PAGE|CanWriteMask);
/*Load page number (upper 4 MSBs of 82C900 reg address) into Tx RAM via data reg. Set for continuous transfer*/	
 MCF5272_WR_QSPI_QDR(imm,PageNumber|CanAutoInc);		

/*Set wrap register for byte write (2 bytes transfers incl. address), starting at top of Tx RAM )*/
 MCF5272_WR_QSPI_QWR(imm,MCF5272_QSPI_QWR_SetPageReg);

/*Enable QSPI transfer*/
 MCF5272_WR_QSPI_QDLYR(imm,MCF5272_QSPI_QDLYR_CanEnable);

/*Poll QSPI finish flag for completion*/
 while (!(MCF5272_RD_QSPI_QIR(imm) & MCF5272_QSPI_QIR_QSPIFinish))
		;
/*Enable flag automatically cleared by QSPI when transfer complete. Added as precaution to prevent*/
/*access to transmit RAM until Tx transfers over.*/
 while (MCF5272_RD_QSPI_QDLYR(imm)& MCF5272_QSPI_QDLYR_CanBusy)
	        ;
}

void mcf5272_qspi_init()
{
/*This routine sets up the QSPI interface for the 82C900 CAN controller*/

MCF5272_IMM *imm = mcf5272_get_immp();

/* Mode reg : set transfer size, clock polarity, clock phase, baud rate*/ 	
	MCF5272_WR_QSPI_QMR(imm,MCF5272_QSPI_QMR_CAN);
/* Delay reg : set CLK delay, delay after transfer, QSPI remains disabled*/
	MCF5272_WR_QSPI_QDLYR(imm,MCF5272_QSPI_QDLYR_CAN);
/* Interrupt reg : disable all interrupts, clear all flags*/
	MCF5272_WR_QSPI_QIR(imm,MCF5272_QSPI_QIR_CAN);
/* Address register : set address pointer to start of command RAM*/
	MCF5272_WR_QSPI_QAR(imm,MCF5272_QSPI_QAR_Command);
/* Data reg :Wr command RAM via QDR. Set QSPI_CS0, Continuous mode, enable 8 bit transfer & delays*/
	MCF5272_WR_QSPI_QDR(imm,MCF5272_QSPI_QDR_CR_CONT);	
	MCF5272_WR_QSPI_QDR(imm,MCF5272_QSPI_QDR_CR_CONT);
	MCF5272_WR_QSPI_QDR(imm,MCF5272_QSPI_QDR_CR_CONT);
	MCF5272_WR_QSPI_QDR(imm,MCF5272_QSPI_QDR_CR_CONT);
	MCF5272_WR_QSPI_QDR(imm,MCF5272_QSPI_QDR_CR_CONT);
	MCF5272_WR_QSPI_QDR(imm,MCF5272_QSPI_QDR_CR_CONT);
	MCF5272_WR_QSPI_QDR(imm,MCF5272_QSPI_QDR_CR_CONT);
	MCF5272_WR_QSPI_QDR(imm,MCF5272_QSPI_QDR_CR_CONT);
	MCF5272_WR_QSPI_QDR(imm,MCF5272_QSPI_QDR_CR_CONT);
	MCF5272_WR_QSPI_QDR(imm,MCF5272_QSPI_QDR_CR_CONT);
	MCF5272_WR_QSPI_QDR(imm,MCF5272_QSPI_QDR_CR_CONT);
	MCF5272_WR_QSPI_QDR(imm,MCF5272_QSPI_QDR_CR_CONT);
	MCF5272_WR_QSPI_QDR(imm,MCF5272_QSPI_QDR_CR_CONT);
	MCF5272_WR_QSPI_QDR(imm,MCF5272_QSPI_QDR_CR_CONT);
	MCF5272_WR_QSPI_QDR(imm,MCF5272_QSPI_QDR_CR_CONT);
	MCF5272_WR_QSPI_QDR(imm,MCF5272_QSPI_QDR_CR_CONT);
}



void QSPI_SendByte(uint16 CanRegAddr, uint8 Data)
{
/* This routine writes a byte of data to a register on the 82C900 CAN controller*/

 MCF5272_IMM *imm = mcf5272_get_immp();

/*Extract 4 MSBs of register address for 82C900 PAGE register*/
 CAN_SetPageReg((uint8)(CanRegAddr>>7));				

/*Point to top of transmit RAM*/
 MCF5272_WR_QSPI_QAR(imm, MCF5272_QSPI_QAR_Tx);
/*Load lower 7 bits of 82C900 reg address into Tx RAM. 8th bit indicates write*/
 MCF5272_WR_QSPI_QDR(imm,(uint8)(CanRegAddr|CanWriteMask));
/*Load data byte into Tx RAM*/
 MCF5272_WR_QSPI_QDR(imm,Data);

/*Set wrap register for byte write (2 bytes transfers incl. address), starting at top of Tx RAM )*/
 MCF5272_WR_QSPI_QWR(imm,MCF5272_QSPI_QWR_SendByte);

/*Enable QSPI transfer*/
 MCF5272_WR_QSPI_QDLYR(imm,MCF5272_QSPI_QDLYR_CanEnable);

/*Poll QSPI finish flag for completion*/
 while (!(MCF5272_RD_QSPI_QIR(imm) & MCF5272_QSPI_QIR_QSPIFinish))
		;

/*Enable flag automatically cleared by QSPI when transfer complete. Added as precaution to prevent*/
/*access to transmit RAM until Tx transfers over.*/
 while (MCF5272_RD_QSPI_QDLYR(imm)& MCF5272_QSPI_QDLYR_CanBusy)
		; 
}

uint8 QSPI_ReadByte(uint16 CanRegAddr)
{

/*This routine reads a byte of data from a register, address CanRegAddr, on the 82C900*/

 MCF5272_IMM *imm = mcf5272_get_immp();

/*Extract 4 MSBs of register address for 82C900 PAGE register*/
 CAN_SetPageReg((uint8)(CanRegAddr>>7));		

/*Point to top of transmit RAM*/
 MCF5272_WR_QSPI_QAR(imm,MCF5272_QSPI_QAR_Tx);
/*Load lower 7 bits of 82C900 address into Tx RAM via the data reg. Indicate Rd.*/
 MCF5272_WR_QSPI_QDR(imm,(uint8)(CanRegAddr&CanReadMask));	
/*Dummy transmission to ensure QSPI clock enabled for receiving byte*/
 MCF5272_WR_QSPI_QDR(imm,(uint8)(CanRegAddr&CanReadMask));		

/*Set wrap register for byte read (2 bytes transfers incl. dummy), starting at top of Tx RAM )*/
 MCF5272_WR_QSPI_QWR(imm,MCF5272_QSPI_QWR_ReadByte);

/*Enable QSPI transfer*/
 MCF5272_WR_QSPI_QDLYR(imm,MCF5272_QSPI_QDLYR_CanEnable);

/*Poll QSPI finish flag for completion*/
 while (!(MCF5272_RD_QSPI_QIR(imm) & MCF5272_QSPI_QIR_QSPIFinish))
		;

/*Enable flag automatically cleared by QSPI when transfer complete. Added as precaution to prevent*/
/*writing to RAM until Tx transfers over.*/
 while (MCF5272_RD_QSPI_QDLYR(imm)& MCF5272_QSPI_QDLYR_CanBusy)
	        ;

/*Point to top of receive RAM*/
 MCF5272_WR_QSPI_QAR(imm,MCF5272_QSPI_QAR_Rx);
/*Read dummy byte received as address transmitted*/
 dummy  = (uint8)MCF5272_RD_QSPI_QDR(imm);
/*Read data returned from the 82C900 register*/
 RxByte = (uint8)MCF5272_RD_QSPI_QDR(imm);

 return RxByte; 
	}

void CAN_MsgObj_Init(uint8 Node, uint8 Msg, uint8 TxRx, uint8 NoBytes, uint8 ID, uint32 IDnum)
{
/*This routine assigns a message object to node, defines no bytes for transfer and sets an ID*/

/*Msg obj tagged invalid to allow update*/
	QSPI_SendByte(CAN_MSG_CTRL + (Msg*0x20), 0x7F);
/*Msg obj interrupt pending flag reset*/
	QSPI_SendByte(CAN_MSG_CTRL + (Msg*0x20), 0xFD);	
/*Msg obj remote request flag reset*/	
	QSPI_SendByte(CAN_MSG_CTRL+ (Msg*0x20),0x7F);
/*Msg obj transmission request flag reset*/
	QSPI_SendByte(CAN_MSG_CTRL+1 + (Msg*0x20), 0xDF);

/*Inhibit transmission for Tx or reset data lost flag for Rx */
	if (TxRx == Tx)
	  QSPI_SendByte(CAN_MSG_CTRL+1 + (Msg*0x20), 0xFD);
	else if (TxRx == Rx)
	  QSPI_SendByte(CAN_MSG_CTRL+1 + (Msg*0x20), 0xF7);	

/*Reset msg obj new data flag */	
	QSPI_SendByte(CAN_MSG_CTRL+1 + (Msg*0x20), 0xFB);

/*Assign Msg obj Node, ID, no bytes*/
	QSPI_SendByte(CAN_MSG_CONFIG + (Msg*0x20), (uint8)(NoBytes<<4|Node|ID|TxRx));

/*Set Msg obj ID*/
if (ID == Stand)
    IDnum = IDnum << 18;
QSPI_SendByte(CAN_MSG_ARB   + (Msg*0x20), (uint8)(IDnum));
QSPI_SendByte(CAN_MSG_ARB+1 + (Msg*0x20), (uint8)(IDnum>>8));	
QSPI_SendByte(CAN_MSG_ARB+2 + (Msg*0x20), (uint8)(IDnum>>16));
QSPI_SendByte(CAN_MSG_ARB+3 + (Msg*0x20), (uint8)(IDnum>>24));

}

void CAN_MsgObj_TxData(uint8 Msg, uint8 NoBytes, uint32 data1, uint32 data2)
{

/*This routine loads the message data register with data to be transmitted */

uint16 n;

/*Split data into bytes and load into the 2x32 bit data register*/
for (n=0; n < NoBytes; n++)
 {
  if ((NoBytes>4)&&(n>=4))
	QSPI_SendByte(CAN_MSG_DAT + n + (Msg*0x20), (uint8)(data2>>(n-4)*8));    	
  else
	QSPI_SendByte(CAN_MSG_DAT + n + (Msg*0x20), (uint8)(data1>>n*8));
 }
}


void CAN_MsgObj_RxData(uint8 Msg, uint8 NoBytes)
{

/*This routine reads the data received for a received message object*/

 uint16 i; 

/*Read received data bytes from the 2x32 bit data registers*/
 for (i=0; i<NoBytes; i++)
  {
   dataRx[i] = QSPI_ReadByte(CAN_MSG_DAT+i+(Msg*0x20));
   printf("data received %d\n", dataRx[i]);
  }
}


void CAN_MsgObj_IntEnable(uint8 Msg, uint8 TxRx, uint8 IntNode)
{
 if (TxRx == Rx)
   {
/*Set msg obj receive interrupt node pointer*/	
    QSPI_SendByte(CAN_MSG_CONFIG+2 + (Msg*0x20), IntNode);
/*Msg obj Rx interrupt enable*/
    QSPI_SendByte(CAN_MSG_CTRL + (Msg*0x20), 0xFB);	    
   }
 else
  {
/*Set msg obj transmit interrupt node pointer*/
    QSPI_SendByte(CAN_MSG_CONFIG+2 + (Msg*0x20), (uint8)(IntNode<<4));	
/*Msg obj Tx interrupt enable*/
    QSPI_SendByte(CAN_MSG_CTRL + (Msg*0x20), 0xEF);	    
 }
}

void CAN_MsgObjRx_Enable(uint8 Msg)
{
/*Msg obj valid */
	QSPI_SendByte(CAN_MSG_CTRL + (Msg*0x20), 0xBF);	

}

void CAN_MsgObjTx_Start(uint8 Msg)
{
/*This routine sets up a message object to begin transmission*/

/*Msg obj CPU update complete, can Tx msg automatically*/
	QSPI_SendByte(CAN_MSG_CTRL+1 + (Msg*0x20), 0xF7);	
/*Msg obj  has new data*/
	QSPI_SendByte(CAN_MSG_CTRL+1 + (Msg*0x20), 0xFE);	
/*Msg obj valid*/
	QSPI_SendByte(CAN_MSG_CTRL + (Msg*0x20), 0xBF);	
/*Msg obj Tx request flag set*/
	QSPI_SendByte(CAN_MSG_CTRL+1 + (Msg*0x20), 0xEF);	
}

void CAN_MsgObjTx_Reset(uint8 Msg)
{
/*This routine resets a message object after transmission*/

/*Msg obj tagged invalid to allow update*/
 QSPI_SendByte(CAN_MSG_CTRL + (Msg*0x20), 0x7F);	
/*Msg obj no new data*/
 QSPI_SendByte(CAN_MSG_CTRL+1 + (Msg*0x20), 0xFD);	
/*Msg obj 0 CPU Update, Tx inhibited*/
 QSPI_SendByte(CAN_MSG_CTRL+1 + (Msg*0x20), 0xFB);	
}


void CAN_MsgObj_IntReset(uint8 Msg)
{
/*The routine resets a message object interrupt*/

 QSPI_SendByte(CAN_MSG_CTRL + (Msg*0x20), 0xFD);	
}


void CAN_Node_Init(void)
{

/*This routine initialises both CAN nodes*/

/*Reset interrupts, stop CAN to initialise*/
	QSPI_SendByte(CAN_ACR, 0x41);		

/*Set CAN data rate at 500 kbit/s : ((0+1)+(6+1)+(7+1)*0.125us)*/
	QSPI_SendByte(CAN_ABTR+1, 0x67);	
	QSPI_SendByte(CAN_ABTR, 0x02);		

/*Synchronise CAN node to bus and enable*/
	QSPI_SendByte(CAN_ACR, 0x00);		


/*Reset interrupts, stop CAN to initialise*/	
	QSPI_SendByte(CAN_BCR, 0x41);	
/*Set CAN data rate at 500 kbit/s : ((0+1)+(6+1)+(7+1)*0.125us)*/
	QSPI_SendByte(CAN_BBTR, 0x02);	
	QSPI_SendByte(CAN_BBTR+1, 0x67);
/*Enable msg obj 1 to be considered as interrupt source*/
	QSPI_SendByte(CAN_BIMR0, 0x02);	
/*Synchronise CAN node to bus and enable*/
	QSPI_SendByte(CAN_BCR, 0x00);		
}


