/*********************************************************************
 *
 * Copyright:
 *	1999-2000 MOTOROLA, INC. All Rights Reserved.  
 *  You are hereby granted a copyright license to use, modify, and
 *  distribute the SOFTWARE so long as this entire notice is
 *  retained without alteration in any modified and/or redistributed
 *  versions, and that such modified versions are clearly identified
 *  as such. No licenses are granted by implication, estoppel or
 *  otherwise under any patents or trademarks of Motorola, Inc. This 
 *  software is provided on an "AS IS" basis and without warranty.
 *
 *  To the maximum extent permitted by applicable law, MOTOROLA 
 *  DISCLAIMS ALL WARRANTIES WHETHER EXPRESS OR IMPLIED, INCLUDING 
 *  IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR
 *  PURPOSE AND ANY WARRANTY AGAINST INFRINGEMENT WITH REGARD TO THE 
 *  SOFTWARE (INCLUDING ANY MODIFIED VERSIONS THEREOF) AND ANY 
 *  ACCOMPANYING WRITTEN MATERIALS.
 * 
 *  To the maximum extent permitted by applicable law, IN NO EVENT
 *  SHALL MOTOROLA BE LIABLE FOR ANY DAMAGES WHATSOEVER (INCLUDING 
 *  WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS 
 *  INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY
 *  LOSS) ARISING OF THE USE OR INABILITY TO USE THE SOFTWARE.   
 * 
 *  Motorola assumes no responsibility for the maintenance and support
 *  of this software
 ********************************************************************/
/*
 * File:		loop.c
 * Purpose:	    Internal Loopback test for FEC
 */

#include "mcf5272.h"
#include "init.h"
#include "eth.h"
#include "nbuf.h"
#include "nif.h"

/********************************************************************/

/* Define an interface to the FEC */
NIF fec_nif;

/* Assign a protocol number for the loop test */
uint16 eth_type = 0x0300;

/* Global variable containing length of data to transmit */
uint16 data_length = 512;

/* External buffers defined in nbuf.c */
extern TXB TxBuffer[];

/* Progress Indicators */
char hash[] = {'-','\\','|','/'};
int ihash = 0;

/* Function Prototypes */
void loop_fill_buffers();
void loop_handler(NIF*, NBUF*);

/********************************************************************/
void main()
{
	NBUF *pNbuf;               
	MCF5272_IMM *imm = mcf5272_get_immp();

	printf("\n\n");
	printf("Running Ethernet Loopback Test\n");
	printf("Built on %s %s\n",__DATE__, __TIME__);
	#if (DEBUG)
		printf("DEBUG prints ON\n");
	#endif

	/* Initialize network device */
	fec_init(&fec_nif);

	/* Write ethernet address in the NIF structure */
	fec_nif.hwa[0] = 0x00;
	fec_nif.hwa[1] = 0xCF;
	fec_nif.hwa[2] = 0x52;
	fec_nif.hwa[3] = 0x72;
	fec_nif.hwa[4] = 0xC3;
	fec_nif.hwa[5] = 0x01;

	/* Bind the Loop Test to the network interface */
	nif_bind_protocol(&fec_nif, eth_type, (void *)loop_handler, (void *)NULL);

	/* Initialize Network Buffers */
	nbuf_init();

	/* Enable FEC interrupts to ColdFire core */
	MCF5272_WR_SIM_ICR3(imm, MCF5272_SIM_ICR_ERX_IL(FEC_LEVEL));

	/* Reset the Ethernet Controller */
	fec_nif.reset(&fec_nif);

	/* Put FEC in internal loopback mode */
	MCF5272_WR_FEC_RCR(imm, MCF5272_RD_FEC_RCR(imm)
		| MCF5272_FEC_RCR_LOOP);

	/* Turn on the FEC */
	fec_nif.start(&fec_nif);

	/* Fill the buffers for this test */
	loop_fill_buffers();

	/* Interrupts take over from here */
	mcf5272_irq_enable();

	/* Transmit until a key is pressed */
	printf("Press any key to stop...\n");
	while (!char_present())
	{
		/* Allocate a buffer to transmit */
		pNbuf = fec_nif.tx_alloc();

		/* Make sure I grabbed a valid Tx buffer */
		if (pNbuf == NULL)
		{
			#if defined(DEBUG_PRINT)
				printf("Loop couldn't allocate Tx buffer\n");
			#endif
		}

		pNbuf->length = data_length;
		
		/* Send this packet to the FEC */
		fec_nif.send(&fec_nif, fec_nif.hwa, fec_nif.hwa, eth_type, pNbuf);
	}
	printf("\nNumber of frames transferred: %d\n",fec_nif.f_rx);
	printf("Number of errors: %d\n",fec_nif.f_rx_err);

	while (1);
}

/********************************************************************/
void
loop_fill_buffers()
{
	/* Fill all the buffers in the TX buffer ring with a unique data pattern */
	uint16 index, pattern, i;

	for (i = 0; i < NUM_TXBDS; i++)
	{
		switch (i % 8)
		{
			/* Load buffers 0 through 3 with a single data patterns */
			case (0):
				memset(&TxBuffer[i].data[ETH_HDR_SIZE],0x55,data_length);
				break;	
			case (1):
				memset(&TxBuffer[i].data[ETH_HDR_SIZE],0xAA,data_length);
				break;
			case (2):
				memset(&TxBuffer[i].data[ETH_HDR_SIZE],0x00,data_length);
				break;
			case (3):
				memset(&TxBuffer[i].data[ETH_HDR_SIZE],0xFF,data_length);
				break;

			/* Buffer[4]: Load increasing walking ones */
			case (4):
				pattern = 1;
				for (index = 0; index < data_length; index++)
				{
					if (pattern == 0x0100)
						pattern = 0x01;	   
					TxBuffer[i].data[index] = (uint8)pattern;
					pattern <<= 1;
				}
				break;
			
			/* Buffer[5]: Load decreasing walking ones */
			case(5):
				pattern = 0x80;
				for (index = 0; index < data_length; index++)
				{
					if (pattern == 0x00)
						pattern = 0x80;
					TxBuffer[i].data[index] = (uint8)pattern;
					pattern >>= 1;
				}		
				break;

			/* Buffer[6]: Load "Increment from 0" pattern */
			case (6):
				for (index = 0; index < data_length; index++)
					TxBuffer[i].data[index] = (uint8) ((index-14)%256);
				break;
			
			/* Buffer[7]: Load "Decrement from 255" pattern */
			case (7):
				for (index = 0; index < data_length; index++)
					TxBuffer[i].data[index] = (uint8)(255- ((index-14)%256));
				break;
		}
	}
}

/********************************************************************/
void 
loop_handler(NIF *fec_nif, NBUF *pNbuf)
{
	/* This is the loop specific RX handler called from the interrupt
	 * receive routine.  This routine gets bound to the loop protocol
	 * (0x0300) by a call to nif_bind_protocol() and is then called in
	 * fec_receive(). This function simply checks to make sure that the
	 * receive buffer matches the transmit buffer.
	 */

	int i;

	/* Compare what I received to what I transmitted */
	for (i = 0; i < (data_length + ETH_HDR_SIZE); i++)
	{
		if (TxBuffer[fec_nif->f_rx % NUM_TXBDS].data[i] != pNbuf->data[i])
		{
			/* Increment reception error count */
			fec_nif->f_rx_err++;
		}
	}
	
	/* Increment reception count */
	fec_nif->f_rx++;

	/* Update progress indicator */
	if (!(fec_nif->f_rx % 200))
	{
		ihash = (ihash + 1) % 4;
		printf("\b%c",hash[ihash]);
	}

	return;
}

/********************************************************************/
