/*********************************************************************
 *
 * Copyright:
 *	1999-2000 MOTOROLA, INC. All Rights Reserved.  
 *  You are hereby granted a copyright license to use, modify, and
 *  distribute the SOFTWARE so long as this entire notice is
 *  retained without alteration in any modified and/or redistributed
 *  versions, and that such modified versions are clearly identified
 *  as such. No licenses are granted by implication, estoppel or
 *  otherwise under any patents or trademarks of Motorola, Inc. This 
 *  software is provided on an "AS IS" basis and without warranty.
 *
 *  To the maximum extent permitted by applicable law, MOTOROLA 
 *  DISCLAIMS ALL WARRANTIES WHETHER EXPRESS OR IMPLIED, INCLUDING 
 *  IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR
 *  PURPOSE AND ANY WARRANTY AGAINST INFRINGEMENT WITH REGARD TO THE 
 *  SOFTWARE (INCLUDING ANY MODIFIED VERSIONS THEREOF) AND ANY 
 *  ACCOMPANYING WRITTEN MATERIALS.
 * 
 *  To the maximum extent permitted by applicable law, IN NO EVENT
 *  SHALL MOTOROLA BE LIABLE FOR ANY DAMAGES WHATSOEVER (INCLUDING 
 *  WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS 
 *  INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY
 *  LOSS) ARISING OF THE USE OR INABILITY TO USE THE SOFTWARE.   
 * 
 *  Motorola assumes no responsibility for the maintenance and support
 *  of this software
 ********************************************************************/

/*
 * File:		icmp.h
 * Purpose:		Handle Internet Control Message Protocol packets.
 *
 * Notes:		See RFC 792 "Internet Control Message Protocol"
 *				for more details.
 *
 *
 * Modifications:
 *
 */

#ifndef _ICMP_H
#define _ICMP_H

/********************************************************************/

typedef struct
{
	uint32	unused;
	uint8	ih_dg;
} icmp_dest_unreachable;
#define ICMP_DEST_UNREACHABLE		(3)	/* type */
#define ICMP_NET_UNREACHABLE		(0)	/* code */
#define ICMP_HOST_UNREACHABLE		(1)
#define ICMP_PROTOCOL_UNREACHABLE	(2)
#define ICMP_PORT_UNREACHABLE		(3)
#define ICMP_FRAG_NEEDED			(4)
#define ICMP_ROUTE_FAILED			(5)

typedef struct
{
	uint32	unused;
	uint8	ih_dg;
} icmp_time_exceeded;
#define ICMP_TIME_EXCEEDED			(11)	/* type */
#define ICMP_TTL_EXCEEDED			(0)		/* code */
#define ICMP_FRAG_TIME_EXCEEDED		(1)

typedef struct
{
	uint8	pointer;
	uint8	unused1;
	uint16	unused2;
	uint8	ih_dg;
} icmp_parameter_problem;
#define ICMP_PARAMETER_PROBLEM		(12)	/* type */
#define ICMP_POINTER				(0)		/* code -- not */

typedef struct
{
	uint32	unused;
	uint8	ih_dg;
} icmp_source_quench;
#define ICMP_SOURCE_QUENCH			(4)		/* type */

typedef struct
{
	uint32	gateway_addr;
	uint8	ih_dg;
} icmp_redirect;
#define ICMP_REDIRECT				(5)		/* type */
#define ICMP_REDIRECT_NET			(0)		/* code */
#define ICMP_REDIRECT_HOST			(1)
#define ICMP_REDIRECT_TOS_NET		(2)
#define ICMP_REDIRECT_TOS_HOST		(3)

typedef struct
{
	uint16	identifier;
	uint16	sequence;
	uint8	data;
} icmp_echo;
#define ICMP_ECHO					(8)		/* type */
#define ICMP_ECHO_REPLY				(0)		/* type */

typedef struct
{
	uint16	identifier;
	uint16	sequence;
} icmp_information;
#define ICMP_INFORMATION_REQUEST	(15)	/* type */
#define ICMP_INFORMATION_REPLY		(16)	/* type */

typedef struct
{
	uint16	identifier;
	uint16	sequence;
	uint32	originate_ts;
	uint32	receive_ts;
	uint32	transmit_ts;
} icmp_timestamp;
#define ICMP_TIMESTAMP				(13)	/* type */
#define ICMP_TIMESTAMP_REPLY		(14)	/* type */

typedef struct
{
	uint8	type;
	uint8	code;
	uint16	chksum;
	union
	{
		icmp_dest_unreachable		dest_unreachable;
		icmp_source_quench			source_quench;
		icmp_redirect				redirect;
		icmp_time_exceeded			time_exceeded;
		icmp_parameter_problem		parameter_problem;
		icmp_timestamp				timestamp;
		icmp_information			information;
		icmp_echo					echo;
	} msg;
} icmp_message;

/********************************************************************/

/* Protocol Header information */
#define ICMP_HDR_OFFSET		(ETH_HDR_SIZE + IP_HDR_SIZE)
#define ICMP_HDR_SIZE		8

void
icmp_handler(NIF *, NBUF *, int);

/********************************************************************/

#endif	/* _ICMP_H */
