/*********************************************************************
 *
 * Copyright:
 *	1999-2000 MOTOROLA, INC. All Rights Reserved.  
 *  You are hereby granted a copyright license to use, modify, and
 *  distribute the SOFTWARE so long as this entire notice is
 *  retained without alteration in any modified and/or redistributed
 *  versions, and that such modified versions are clearly identified
 *  as such. No licenses are granted by implication, estoppel or
 *  otherwise under any patents or trademarks of Motorola, Inc. This 
 *  software is provided on an "AS IS" basis and without warranty.
 *
 *  To the maximum extent permitted by applicable law, MOTOROLA 
 *  DISCLAIMS ALL WARRANTIES WHETHER EXPRESS OR IMPLIED, INCLUDING 
 *  IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR
 *  PURPOSE AND ANY WARRANTY AGAINST INFRINGEMENT WITH REGARD TO THE 
 *  SOFTWARE (INCLUDING ANY MODIFIED VERSIONS THEREOF) AND ANY 
 *  ACCOMPANYING WRITTEN MATERIALS.
 * 
 *  To the maximum extent permitted by applicable law, IN NO EVENT
 *  SHALL MOTOROLA BE LIABLE FOR ANY DAMAGES WHATSOEVER (INCLUDING 
 *  WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS 
 *  INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY
 *  LOSS) ARISING OF THE USE OR INABILITY TO USE THE SOFTWARE.   
 * 
 *  Motorola assumes no responsibility for the maintenance and support
 *  of this software
 ********************************************************************/

/*
 * File:		main.c
 * Purpose:	    TFTP Network Stack user interface
 */

#include "mcf5272.h"
#include "init.h"
#include "nif.h"
#include "ip.h"

/********************************************************************/
/* Define an interface to the FEC */
NIF fec_nif;

static const char PROMPT[] = "tftp> ";

/********************************************************************/
void
main (void)
{
	IP_ADDR client;
	IP_ADDR gateway;
	IP_ADDR netmask;
	IP_ADDR server;

	printf("\n\n");
	printf("Running TFTP Network Stack\n");
	printf("Built on %s %s\n",__DATE__, __TIME__);
	#if (DEBUG)
		printf("DEBUG prints ON\n");
	#endif

	printf("Enter \"?\" for help.\n");

	while (1)
	{
		printf(PROMPT);
		run_cmd();
	}
}

/********************************************************************/
void
run_cmd (void)
{
	int argc;
	char *argv[ + 1];	/* one extra for null terminator */

	get_history_line(cmdline1);

	if (!(argc = make_argv(cmdline1,argv)))
	{
		/* no command entered, just a blank line */
		strcpy(cmdline1,cmdline2);
		argc = make_argv(cmdline1,argv);
	}
	cmdline2[0] = '\0';

	if (argc)
	{
		int i;
		/*
		 * First try for an exact match on command name
		 */
		for (i = 0; i < UIF_NUM_CMD; i++)
		{
			if (strcasecmp(UIF_CMDTAB[i].cmd,argv[0]) == 0)
			{
				if (((argc-1) >= UIF_CMDTAB[i].min_args) &&
					((argc-1) <= UIF_CMDTAB[i].max_args))
				{
					if (UIF_CMDTAB[i].flags & UIF_CMD_FLAG_REPEAT)
					{
						strcpy(cmdline2,argv[0]);
					}
					UIF_CMDTAB[i].func(argc,argv);
					return;
				}
				else
				{
					printf(SYNTAX,argv[0]);
					return;
				}
			}
		}

		/*
		 * Now try for short-hand match on command name
		 */
		for (i = 0; i < UIF_NUM_CMD; i++)
		{
			if (strncasecmp(UIF_CMDTAB[i].cmd,argv[0],
				UIF_CMDTAB[i].unique) == 0)
			{
				if (((argc-1) >= UIF_CMDTAB[i].min_args) &&
					((argc-1) <= UIF_CMDTAB[i].max_args))
				{
					if (UIF_CMDTAB[i].flags & UIF_CMD_FLAG_REPEAT)
					{
						strcpy(cmdline2,argv[0]);
					}
					UIF_CMDTAB[i].func(argc,argv);
					return;
				}
				else
				{
					printf(SYNTAX,argv[0]);
					return;
				}
			}
		}
		printf(INVCMD,argv[0]);
		printf(HELPMSG);
	}
}

/********************************************************************/
char *
get_user_input (char *userline)
{
	char line[UIF_MAX_LINE];
	int pos, ch, i, curr_hist, repeat;

	curr_hist = -1;	/* invalid */

	repeat = FALSE;
	pos = 0;

	ch = (int)board_getchar();
	while (	(ch != 0x0D /* CR */) &&
			(ch != 0x0A /* LF/NL */) &&
			(pos < UIF_MAX_LINE))
	{
		switch (ch)
		{
			case 0x08:		/* Backspace */
			case 0x7F:		/* Delete */
				if (pos > 0)
				{
					pos -= 1;
					board_putchar(0x08);	/* backspace */
					board_putchar(' ');
					board_putchar(0x08);	/* backspace */
				}
				break;
			case CTRL_U:
			case CTRL_D:
			case CTRL_R:
				for (i = 0; i < pos; ++i)
				{
					board_putchar(0x08);	/* backspace */
					board_putchar(' ');
					board_putchar(0x08);	/* backspace */
				}
				if (ch == CTRL_U)
				{
					pos = history_up(&curr_hist,line);
					break;
				}
				if (ch == CTRL_D)
				{
					pos = history_down(&curr_hist,line);
					break;
				}
				if (ch == CTRL_R)
				{
					pos = history_repeat(line);
					repeat = TRUE;
				}
				break;
			default:
				if ((pos+1) < UIF_MAX_LINE)
				{
					/* only printable characters */
					if ((ch > 0x1f) && (ch < 0x80))
					{
						line[pos++] = (char)ch;
						board_putchar((char)ch);
					}
				}
				break;
		}

		if (repeat)
		{
			break;
		}
		ch = (int)board_getchar();
	}
	line[pos] = '\0';
	board_putchar(0x0D);	/* CR */
	board_putchar(0x0A);	/* LF */

	if ((strlen(line) != 0) && !repeat)
	{
		history_add(line);
	}
	strcpy(userline,line);

	return userline;
}

/********************************************************************/
