/*********************************************************************
 *
 * Copyright:
 *	1999-2000 MOTOROLA, INC. All Rights Reserved.  
 *  You are hereby granted a copyright license to use, modify, and
 *  distribute the SOFTWARE so long as this entire notice is
 *  retained without alteration in any modified and/or redistributed
 *  versions, and that such modified versions are clearly identified
 *  as such. No licenses are granted by implication, estoppel or
 *  otherwise under any patents or trademarks of Motorola, Inc. This 
 *  software is provided on an "AS IS" basis and without warranty.
 *
 *  To the maximum extent permitted by applicable law, MOTOROLA 
 *  DISCLAIMS ALL WARRANTIES WHETHER EXPRESS OR IMPLIED, INCLUDING 
 *  IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR
 *  PURPOSE AND ANY WARRANTY AGAINST INFRINGEMENT WITH REGARD TO THE 
 *  SOFTWARE (INCLUDING ANY MODIFIED VERSIONS THEREOF) AND ANY 
 *  ACCOMPANYING WRITTEN MATERIALS.
 * 
 *  To the maximum extent permitted by applicable law, IN NO EVENT
 *  SHALL MOTOROLA BE LIABLE FOR ANY DAMAGES WHATSOEVER (INCLUDING 
 *  WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS 
 *  INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY
 *  LOSS) ARISING OF THE USE OR INABILITY TO USE THE SOFTWARE.   
 * 
 *  Motorola assumes no responsibility for the maintenance and support
 *  of this software
 ********************************************************************/

/*
 * File:		main.c
 * Purpose:	    TFTP Network Stack user interface
 */

#include "mcf5272.h"
#include "init.h"
#include "nif.h"
#include "arp.h"
#include "ip.h"
#include "udp.h"
#include "tftp.h"
#include "timer.h"
#include "tftp_main.h"

/********************************************************************/

/* Define an interface to the FEC */
NIF			fec_nif;
IP_INFO		ip_info;
ARP_INFO	arp_info;

static char input[MAX_LINE];
static const char PROMPT[] = "TFTP> ";
static const char HELPMSG[] = "\nEnter 'help' for help.\n\n";
static const char SYNTAX[] = "Error: Invalid syntax for: %s\n";
static const char INVCMD[] = "Error: No such command: %s\n";
static const char INVALUE[] = "Error: Invalid value: %s\n";
static const char HELPFORMAT[] = "%8s %-30s %s %s\n";
static const char INVOPT[] = "Error:  Invalid set/show option: %s\n";
static const char OPTFMT[] = "%12s: ";
static const char MACFMT[] = "%02X:%02X:%02X:%02X:%02X:%02X\n";
static const char SETERR[] = "Error: Invalid MAC address: %s\n";
static const char IPFMT[] = "%d.%d.%d.%d\n";
static const char SETERR1[] = "Error: Invalid IP address: %s\n";
static const USERSPACE = 0x20000;
static IP_ADDR client = {163,2,28,158};
static IP_ADDR gateway = {163,2,31,254};
static IP_ADDR netmask = {255,255,248,0};
static IP_ADDR server = {163,2,28,157};
static unsigned char mac[6] = {0x0, 0xCF, 0x52, 0x72, 0xC3, 0x01};
static char filename[MAX_FN] = "test.bin";
uint32 md_last_address = 0x20000;

CMD CMDTAB[] =
{
	{"help",  0, 0, help,  "Display this help message", ""},
	{"read",  0, 1, read,  "Read from TFTP host", "<filename>"},
	{"md",	  0, 1, md,    "Memory display", "<address>"},
	{"set",   0, 2, set,   "Set parameter", "option value"},
	{"show",  0, 0, show,  "Show parameters", ""},
	{"write", 1, 2, write, "Write to TFTP host", "bytes <filename>"},
	{"quit",  0, 0, quit,  "Exit TFTP", " "}
};
const int NUM_CMD = sizeof(CMDTAB)/sizeof(CMD);

SETCMD SETCMDTAB[] =
{
	{"server",   set_server,   "<server IP address>"},
	{"client",   set_client,   "<board IP address>"},
	{"gateway",  set_gateway,  "<gateway IP address>"},
	{"netmask",  set_netmask,  "<netmask IP address>"},
	{"mac",      set_mac,      "<ethernet address>"},
	{"filename", set_filename, "<file to transfer>"}
};
const int NUM_SETCMD = sizeof(SETCMDTAB)/sizeof(SETCMD);

/********************************************************************/
void
main (void)
{
	printf("\n\n");
	printf("Running TFTP Network Stack\n");
	printf("Built on %s %s\n",__DATE__, __TIME__);
	#if (DEBUG)
		printf("DEBUG prints ON\n");
	#endif

	printf(HELPMSG);

	while (1)
	{
		printf(PROMPT);
		run_cmd();
	}
}

/********************************************************************/
static void
get_user_input (char *userline)
{
	char line[MAX_LINE];
	int pos, ch;

	pos = 0;

	ch = (int)in_char();
	while (	(ch != 0x0D /* CR */) &&
			(ch != 0x0A /* LF/NL */) &&
			(pos < MAX_LINE))
	{
		switch (ch)
		{
			case 0x08:		/* Backspace */
			case 0x7F:		/* Delete */
				if (pos > 0)
				{
					pos -= 1;
					out_char(0x08);	/* backspace */
					out_char(' ');
					out_char(0x08);	/* backspace */
				}
				break;
			default:
				if ((pos+1) < MAX_LINE)
				{
					/* only printable characters */
					if ((ch > 0x1f) && (ch < 0x80))
					{
						line[pos++] = (char)ch;
						out_char((char)ch);
					}
				}
				break;
		}

		ch = (int)in_char();
	}
	
	out_char(0x0D);	/* CR */
	out_char(0x0A);	/* LF */
	
	if (!pos && (strncasecmp(userline,"md",2) == 0))
	{
		/* Allow 'md' command to be repeated */
	}
	else
	{
		line[pos] = '\0';
		strcpy(userline,line);
	}
}

/********************************************************************/
static int
make_argv (char *cmdline, char *argv[])
{
	int argc, i, in_text;

	/* break cmdline into strings and argv */
	/* it is permissible for argv to be NULL, in which case */
	/* the purpose of this routine becomes to count args */
	argc = 0;
	i = 0;
	in_text = FALSE;
	while (cmdline[i] != '\0')	/* getline() must place 0x00 on end */
	{
		if (((cmdline[i] == ' ')   ||
			 (cmdline[i] == '\t')) )
		{
			if (in_text)
			{
				/* end of command line argument */
				cmdline[i] = '\0';
				in_text = FALSE;
			}
			else
			{
				/* still looking for next argument */
				
			}
		}
		else
		{
			/* got non-whitespace character */
			if (in_text)
			{
			}
			else
			{
				/* start of an argument */
				in_text = TRUE;
				if (argc < MAX_ARGS)
				{
					if (argv != NULL)
						argv[argc] = &cmdline[i];
					argc++;
				}
				else
					/*return argc;*/
					break;
			}

		}
		i++;	/* proceed to next character */
	}
	if (argv != NULL)
		argv[argc] = NULL;
	return argc;
}

/********************************************************************/
static uint32
get_value (char *s, int *success, int base)
{
	uint32 value;
	char *p;

	value = strtoul(s,&p,base);
	if ((value == 0) && (p == s))
	{
		*success = FALSE;
		return 0;
	}
	else
	{
		*success = TRUE;
		return value;
	}
}

/********************************************************************/
void
run_cmd (void)
{
	int argc;
	char *argv[MAX_ARGS + 1];	/* One extra for NULL terminator */

	get_user_input(input);

	argc = make_argv(input,argv);

	if (argc)
	{
		int i;
		for (i = 0; i < NUM_CMD; i++)
		{
			if (strcasecmp(CMDTAB[i].cmd,argv[0]) == 0)
			{
				if (((argc-1) >= CMDTAB[i].min_args) &&
					((argc-1) <= CMDTAB[i].max_args))
				{
					CMDTAB[i].func(argc,argv);
					return;
				}
				else
				{
					printf(SYNTAX,argv[0]);
					return;
				}
			}
		}
		printf(INVCMD,argv[0]);
		printf(HELPMSG);
	}
}

/********************************************************************/
void
help (int argc, char **argv)
{
	int index;
	
	(void)argc;
	(void)argv;
	
	printf("\n");
	for (index = 0; index < NUM_CMD; index++)
	{
		printf(HELPFORMAT,
			CMDTAB[index].cmd,
			CMDTAB[index].description,
			CMDTAB[index].cmd,
			CMDTAB[index].syntax);
	}
	printf("\n");
}

/********************************************************************/
void
read (int argc, char **argv)
{
	MCF5272_IMM *imm = mcf5272_get_immp();
	uint32 addr = USERSPACE;
	char *fn;
	int ch;

	if (argc == 2)
		fn = (char *)&argv[1][0];
	else
		fn = filename;
	
	/* Initialize the timer for network use */
	timer_init(imm, TIMER_NETWORK, TIMER_NETWORK_PERIOD, \
					SYSTEM_CLOCK, TIMER_NETWORK_LEVEL);

	/* Enable FEC interrupts to ColdFire core */
	MCF5272_WR_SIM_ICR3(imm, MCF5272_SIM_ICR_ERX_IL(FEC_LEVEL));

	/* Initialize network device */
	fec_init(&fec_nif);

	/* Write ethernet address in the NIF structure */
	fec_nif.hwa[0] = mac[0];
	fec_nif.hwa[1] = mac[1];
	fec_nif.hwa[2] = mac[2];
	fec_nif.hwa[3] = mac[3];
	fec_nif.hwa[4] = mac[4];
	fec_nif.hwa[5] = mac[5];

	/* Initialize Network Buffers */
	nbuf_init();

	/* Initialize ARP */
	arp_init(&arp_info);
	nif_bind_protocol(&fec_nif,FRAME_ARP,(void *)arp_handler,(void *)&arp_info);

	/* Initialize IP */
	ip_init(&ip_info,client,gateway,netmask);
	nif_bind_protocol(&fec_nif,FRAME_IP,(void *)ip_handler,(void *)&ip_info);
	
	/* Initialize UDP */
	udp_init();

	printf("Reading %s from %d.%d.%d.%d to 0x%08X\n",
		fn, server[0], server[1], server[2], server[3], addr);

	/* Open the TFTP connection */
	if (tftp_read(&fec_nif, fn, server))
	{
		while (1)
		{
			ch = tftp_in_char();
			if (ch == -1)
			{
				/* We're done */
				break;
			}
			else
			{
				*(unsigned char *)addr++ = (unsigned char)ch;
			}
		}
		
		/* Close the TFTP connection */
		tftp_end(TRUE);
	}
}

/********************************************************************/
void
write (int argc, char **argv)
{
	MCF5272_IMM *imm = mcf5272_get_immp();
	uint32 bytes;
	int success;
	char *fn;

	bytes = get_value(argv[1],&success,10);
	if (!success)
	{
		printf(INVALUE,argv[1]);
		return;
	}

	if (argc == 3)
		fn = (char *)&argv[2][0];
	else
		fn = filename;
	
	/* Initialize the timer for network use */
	timer_init(imm, TIMER_NETWORK, TIMER_NETWORK_PERIOD, \
					SYSTEM_CLOCK, TIMER_NETWORK_LEVEL);

	/* Enable FEC interrupts to ColdFire core */
	MCF5272_WR_SIM_ICR3(imm, MCF5272_SIM_ICR_ERX_IL(FEC_LEVEL));

	/* Initialize network device */
	fec_init(&fec_nif);

	/* Write ethernet address in the NIF structure */
	fec_nif.hwa[0] = mac[0];
	fec_nif.hwa[1] = mac[1];
	fec_nif.hwa[2] = mac[2];
	fec_nif.hwa[3] = mac[3];
	fec_nif.hwa[4] = mac[4];
	fec_nif.hwa[5] = mac[5];

	/* Initialize Network Buffers */
	nbuf_init();

	/* Initialize ARP */
	arp_init(&arp_info);
	nif_bind_protocol(&fec_nif,FRAME_ARP,(void *)arp_handler,(void *)&arp_info);

	/* Initialize IP */
	ip_init(&ip_info,client,gateway,netmask);
	nif_bind_protocol(&fec_nif,FRAME_IP,(void *)ip_handler,(void *)&ip_info);
	
	/* Initialize UDP */
	udp_init();

	printf("Sending data from 0x%08X to 0x%08X to %d.%d.%d.%d\n",
			USERSPACE, USERSPACE + bytes, 
			server[0], server[1], server[2], server[3]);
	printf("Remote filename: %s\n", fn);

	tftp_write(&fec_nif, fn, server, USERSPACE, USERSPACE + bytes);
}

/********************************************************************/
void
quit (int argc, char **argv)
{
	(void)argc;
	(void)argv;

	printf("Exiting TFTP\n");
	while (1)
		;
}

/********************************************************************/
static void
dump_mem (uint32 begin, uint32 end)
{
	uint32 data;
	uint32 curr;
	char line[16];
	char *lcur;
	int i, ch;

	curr = begin;

	do
	{
		printf("%08X:  ",curr);
		lcur = line;
		i = 0;
		while (i < 16)
		{
			data = *((uint32 *)curr);
			printf("%08X ", data);
			*(uint32 *)lcur = data;
			curr += 4;
			lcur += 4;
			i += 4;
		}

		for (i = 0; i < 16; i++)
		{
			ch = line[i];
			if ((ch >= ' ') && (ch <= '~'))
				printf("%c",ch);
			else
				printf(".");
		}
		printf("\n");

	} while (curr < end);
}

/********************************************************************/
void
md (int argc, char **argv)
{
	int success;
	uint32 begin, end;

	if (argc > 1)
	{
		begin = get_value(argv[1],&success,16);
		if (!success)
		{
			printf(INVALUE,argv[1]);
			return;
		}
		begin = begin & 0xFFFFFFFC;
		end = begin + (8 * 16);
	}
	else
	{
		begin = md_last_address;
		end = begin + (8 * 16);
	}

	dump_mem(begin,end);
	md_last_address = end;
}

/********************************************************************/
static int
ip_convert_address (char *ipstring, uint8 ipaddr[])
{
	int i,j,k,l;

	l = strlen(ipstring);

	for (i = 0; i < l; i++)
	{
		if ((ipstring[i] != '.') && ((ipstring[i] < '0') ||
			(ipstring[i] > '9')))
		{
			return 0;
		}
	}
	/* the characters in the string are at least valid */
	j = 0;
	k = 0;
	for (i = 0; i < l; i++)
	{
		if (ipstring[i] != '.')
		{
			if  (++j > 3)	/* number of digits in portion */
				return 0;
		}
		else
		{
			if (++k > 3)	/* number of periods */
				return 0;

			if (j == 0)		/* number of digits in portion */
				return 0;
			else
				j = 0;
		}
	}

	/* at this point, all the pieces are good, form ip_address */
	k = 0; j = 0;
	for (i = 0; i < l; i++)
	{
		if (ipstring[i] != '.')
		{
			k = (k * 10) + ipstring[i] - '0';
		}
		else
		{
			ipaddr[j++] = (unsigned char)k;
			k = 0;
		}
	}
	ipaddr[j] = (uint8)k;
	return 1;
}

/********************************************************************/
static int
mac_convert_address (char *macstring, unsigned char macaddr[])
{
	uint8 i, digits, colons;
	int success, length;
	char *str;

	length = strlen(macstring);
	str = macstring;
	digits = colons = 0;

	for (i = 0; i < length; i++)
	{
		if (macstring[i] != ':')
		{
			if (++digits > 3) 
				return 0;
		}
		else
		{
			if (++colons > 5) 
				return 0;
			else if (digits == 0) 
				return 0;
			else
			{
				digits = 0;
				macstring[i] = (uint8)NULL;
				macaddr[colons-1] = (uint8)get_value(str,&success,16);
				if (!success)	
					return 0;
				str = &macstring[i+1];
			}
		}
	}
	if (colons != 5) 
		return 0;

	macaddr[colons] = (uint8)get_value(str,&success,16);

	if (!success) 
		return 0;

	return 1;
}

/********************************************************************/
void
set_server (int argc, char **argv)
{
	(void)argc;

	if (argv[2] == NULL)
	{
		printf(IPFMT, server[0], server[1], server[2], server[3]);
		return;
	}
	if (!ip_convert_address(argv[2],server))
	{
		printf(SETERR1,argv[2]);
		return;
	}
}

/********************************************************************/
void
set_client (int argc, char **argv)
{
	(void)argc;

	if (argv[2] == NULL)
	{
		printf(IPFMT, client[0], client[1], client[2], client[3]);
		return;
	}
	if (!ip_convert_address(argv[2],client))
	{
		printf(SETERR1,argv[2]);
		return;
	}
}

/********************************************************************/
void
set_gateway (int argc, char **argv)
{
	(void)argc;

	if (argv[2] == NULL)
	{
		printf(IPFMT, gateway[0], gateway[1], gateway[2], gateway[3]);
		return;
	}
	if (!ip_convert_address(argv[2],gateway))
	{
		printf(SETERR1,argv[2]);
		return;
	}
}

/********************************************************************/
void
set_netmask (int argc, char **argv)
{
	(void)argc;

	if (argv[2] == NULL)
	{
		printf(IPFMT, netmask[0], netmask[1], netmask[2], netmask[3]);
		return;
	}
	if (!ip_convert_address(argv[2],netmask))
	{
		printf(SETERR1,argv[2]);
		return;
	}
}

/********************************************************************/
void
set_filename (int argc, char **argv)
{
	(void)argc;

	if (argv[2] == NULL)
	{
		printf("%s\n",filename);
		return;
	}
	strncpy(filename,argv[2],MAX_FN);
}

/********************************************************************/
void
set_mac (int argc, char **argv)
{
	(void)argc;

	if (argv[2] == NULL)
	{
		printf(MACFMT, mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
		return;
	}
	if (!mac_convert_address(argv[2],mac))
	{
		printf(SETERR,argv[2]);
		return;
	}
}

/********************************************************************/
void
set (int argc, char **argv)
{
	int index;

	if (argc == 1)
	{
		printf("\n");
		printf("Valid 'set' options:\n");
		for (index = 0; index < NUM_SETCMD; ++index)
		{
			printf(OPTFMT,SETCMDTAB[index].option);
			printf("%s\n",SETCMDTAB[index].syntax);
		}
		printf("\n");
		return;
	}

	if (argc != 3)
	{
		printf("Error: Invalid argument list\n");
		return;
	}

	for (index = 0; index < NUM_SETCMD; index++)
	{
		if (strcasecmp(SETCMDTAB[index].option,argv[1]) == 0)
		{
			SETCMDTAB[index].func(argc,argv);
			return;
		}
	}
	printf(INVOPT,argv[1]);
}

/********************************************************************/
void
show (int argc, char **argv)
{
	int index;

	/* Show all Option settings */
	argc = 2;
	argv[2] = NULL;
	printf("\n");
	for (index = 0; index < NUM_SETCMD; index++)
	{
		printf(OPTFMT,SETCMDTAB[index].option);
		SETCMDTAB[index].func(argc,argv);
	}
	printf("\n");
}

/********************************************************************/
