/*********************************************************************
 *
 * Copyright:
 *	1999-2000 MOTOROLA, INC. All Rights Reserved.  
 *  You are hereby granted a copyright license to use, modify, and
 *  distribute the SOFTWARE so long as this entire notice is
 *  retained without alteration in any modified and/or redistributed
 *  versions, and that such modified versions are clearly identified
 *  as such. No licenses are granted by implication, estoppel or
 *  otherwise under any patents or trademarks of Motorola, Inc. This 
 *  software is provided on an "AS IS" basis and without warranty.
 *
 *  To the maximum extent permitted by applicable law, MOTOROLA 
 *  DISCLAIMS ALL WARRANTIES WHETHER EXPRESS OR IMPLIED, INCLUDING 
 *  IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR
 *  PURPOSE AND ANY WARRANTY AGAINST INFRINGEMENT WITH REGARD TO THE 
 *  SOFTWARE (INCLUDING ANY MODIFIED VERSIONS THEREOF) AND ANY 
 *  ACCOMPANYING WRITTEN MATERIALS.
 * 
 *  To the maximum extent permitted by applicable law, IN NO EVENT
 *  SHALL MOTOROLA BE LIABLE FOR ANY DAMAGES WHATSOEVER (INCLUDING 
 *  WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS 
 *  INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY
 *  LOSS) ARISING OF THE USE OR INABILITY TO USE THE SOFTWARE.   
 * 
 *  Motorola assumes no responsibility for the maintenance and support
 *  of this software
 ********************************************************************/

/*
 * File:		timer.h
 * Purpose:		routines for accessing integrated timer modules
 *
 * Notes:		
 *
 *
 * Modifications:
 *
 */

#ifndef _TIMER_H
#define _TIMER_H

/********************************************************************/

typedef struct
{
	MCF5272_IMM *immp;				/* internal memory map pointer */
	float	period;					/* tick period in nano-seconds */
	uint16	then;					/* previous TCN value */
	uint16	now;					/* most recent TCN value */
	uint32	timeouts;				/* count of timer reference reaches */
	uint32	reference;				/* number of timeouts desired */
	uint32	level;					/* Interrupt priority for this channel */
	uint16	tmr;					/* Timer Mode Register */
	uint16	trr;					/* Timer Reference Register */
	uint8	channel;				/* which channel is this structure for? */
} mcf5272_timer;

/********************************************************************/

uint32 
timer_init(
	MCF5272_IMM *,		/* immp	- Internal memory map pointer (MBAR)		*/
	uint8,				/* channel - Timer to initialize (0->3)				*/
	float,				/* period - Tick period in nano-seconds				*/
						/*    Min = 1/sysclock, Max = (256 * 16)/sysclock)	*/
						/*    If period == 0, disable timer					*/
	float,				/* sysclock	- System bus clock						*/
	uint32				/* level - Interrupt priority level					*/
	); 

void
timer_stop(uint8);

uint32 
timer_read(uint8);

uint32 
timer_set_secs(uint8, uint32);

uint32 
timer_get_reference(uint8);

/********************************************************************/

/* Vector numbers for all the timer channels */
#define TIMER_VECTOR(a)		(a + 69)

/********************************************************************/

#endif