/*********************************************************************
 *
 * Copyright:
 *	1999-2000 MOTOROLA, INC. All Rights Reserved.  
 *  You are hereby granted a copyright license to use, modify, and
 *  distribute the SOFTWARE so long as this entire notice is
 *  retained without alteration in any modified and/or redistributed
 *  versions, and that such modified versions are clearly identified
 *  as such. No licenses are granted by implication, estoppel or
 *  otherwise under any patents or trademarks of Motorola, Inc. This 
 *  software is provided on an "AS IS" basis and without warranty.
 *
 *  To the maximum extent permitted by applicable law, MOTOROLA 
 *  DISCLAIMS ALL WARRANTIES WHETHER EXPRESS OR IMPLIED, INCLUDING 
 *  IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR
 *  PURPOSE AND ANY WARRANTY AGAINST INFRINGEMENT WITH REGARD TO THE 
 *  SOFTWARE (INCLUDING ANY MODIFIED VERSIONS THEREOF) AND ANY 
 *  ACCOMPANYING WRITTEN MATERIALS.
 * 
 *  To the maximum extent permitted by applicable law, IN NO EVENT
 *  SHALL MOTOROLA BE LIABLE FOR ANY DAMAGES WHATSOEVER (INCLUDING 
 *  WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS 
 *  INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY
 *  LOSS) ARISING OF THE USE OR INABILITY TO USE THE SOFTWARE.   
 * 
 *  Motorola assumes no responsibility for the maintenance and support
 *  of this software
 ********************************************************************/

/*
 * File:		udp.c
 * Purpose:		User Datagram Protocol driver
 *
 * Notes:
 *
 * Modifications:
 *
 */

#include "mcf5272.h"
#include "init.h"
#include "nif.h"
#include "ip.h"
#include "udp.h"

/********************************************************************/
typedef struct
{
	uint16 port;
	void (*handler)(NIF *, NBUF *, int);

} UDP_BOUND_PORT;

/********************************************************************/

#define UDP_MAX_PORTS	(5)		/* plenty for this implementation */

static UDP_BOUND_PORT
udp_port_table[UDP_MAX_PORTS];

static uint16
udp_port;

/********************************************************************/
void
udp_init (void)
{
	int index;

	for (index = 0; index < UDP_MAX_PORTS; ++index)
	{
		udp_port_table[index].port = 0;
	}

	udp_port = DEFAULT_UDP_PORT;	/* next free port */
}

/********************************************************************/
void
udp_prime_port (uint16 init_port)
{
	udp_port = init_port;
}


/********************************************************************/
void
udp_bind_port (uint16 port,	void (*handler)(NIF *, NBUF *, int))
{
	int index;

	for (index = 0; index < UDP_MAX_PORTS; ++index)
	{
		if (udp_port_table[index].port == 0)
		{
			udp_port_table[index].port = port;
			udp_port_table[index].handler = handler;
			return;
		}
	}
}

/********************************************************************/
void
udp_free_port (uint16 port)
{
	int index;

	for (index = 0; index < UDP_MAX_PORTS; ++index)
	{
		if (udp_port_table[index].port == port)
		{
			udp_port_table[index].port = 0;
			return;
		}
	}
}

/********************************************************************/
static void *
udp_port_handler (uint16 port)
{
	int index;

	for (index = 0; index < UDP_MAX_PORTS; ++index)
	{
		if (udp_port_table[index].port == port)
		{
			return (void *)udp_port_table[index].handler;
		}
	}
	return NULL;
}

/********************************************************************/
uint16
udp_obtain_free_port (void)
{
	uint16 port;

	port = udp_port;
	if (--udp_port <= 255)
		udp_port = DEFAULT_UDP_PORT;

	return port;
}

/********************************************************************/
int
udp_send ( NIF *nif, uint8 *dest, int sport, int dport, NBUF *pNbuf)
{
	/****************************************************************
	 This function takes data and creates a UDP frame and
	 passes it onto the IP layer.
	*****************************************************************/

	udp_frame_hdr	*udpframe;

	if ((pNbuf->length < 0) || (pNbuf->length > 1500))
		return 0;
	
	udpframe = (udp_frame_hdr *)&pNbuf->data[UDP_HDR_OFFSET];
	
	/* Set UDP source port */
	udpframe->src_port = (uint16)sport;

	/* Set UDP destination port */
	udpframe->dest_port = (uint16)dport;

	/* Set length */
	udpframe->length = (uint16)(pNbuf->length + 8);

	/* No checksum calcualation needed */
	udpframe->chksum = (uint16)0;

	/* Add the length of the UDP packet to the total length of the packet */
	pNbuf->length += 8;

	return ( ip_send(nif, 
	                 dest,
					 ip_get_myip(nif_get_protocol_info(nif,FRAME_IP)),
					 IP_PROTO_UDP, 
					 pNbuf));
}

/********************************************************************/
void
udp_handler (NIF *nif, NBUF *pNbuf, int hdr_offset)
{
	/* This function handles incoming UDP packets */

	udp_frame_hdr	*udpframe;
	int (*handler)(NIF *, NBUF *, int);

	udpframe = (udp_frame_hdr *)&pNbuf->data[hdr_offset];

	/* Subtract the length of the UDP header from the total length */
	pNbuf->length -= 8;
	
	/****************************************************************
	 Traverse the list of bound ports to see if there is a higher
	 level protocol to pass the packet on to 
	*****************************************************************/
	if ((handler = udp_port_handler(UDP_DEST(udpframe))) != NULL)
		handler(nif, pNbuf, hdr_offset+UDP_HDR_SIZE);

	return;
}

/********************************************************************/
