/********************************************************************
 *
 * Copyright:
 *	1999-2000 MOTOROLA, INC. All Rights Reserved.  
 *  You are hereby granted a copyright license to use, modify, and
 *  distribute the SOFTWARE so long as this entire notice is
 *  retained without alteration in any modified and/or redistributed
 *  versions, and that such modified versions are clearly identified
 *  as such. No licenses are granted by implication, estoppel or
 *  otherwise under any patents or trademarks of Motorola, Inc. This 
 *  software is provided on an "AS IS" basis and without warranty.
 *
 *  To the maximum extent permitted by applicable law, MOTOROLA 
 *  DISCLAIMS ALL WARRANTIES WHETHER EXPRESS OR IMPLIED, INCLUDING 
 *  IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR
 *  PURPOSE AND ANY WARRANTY AGAINST INFRINGEMENT WITH REGARD TO THE 
 *  SOFTWARE (INCLUDING ANY MODIFIED VERSIONS THEREOF) AND ANY 
 *  ACCOMPANYING WRITTEN MATERIALS.
 * 
 *  To the maximum extent permitted by applicable law, IN NO EVENT
 *  SHALL MOTOROLA BE LIABLE FOR ANY DAMAGES WHATSOEVER (INCLUDING 
 *  WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS 
 *  INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY
 *  LOSS) ARISING OF THE USE OR INABILITY TO USE THE SOFTWARE.   
/*
 * File:		main.c
 * Purpose:		MCF5272 CAN reference design. Assigns a message object to each CAN node on the 82C900,
 *			one for receive and one for transmit. Both message objects are given the same ID and the receive 
 *			message object triggers an interrupt on receiving the data transmitted. The interrupt handler 
 *			routine captures the data, modifies the transmission data and restarts transmission. The data 
* *			received is ouput over the terminal uart on the M5272C3 board.
 */

#include "mcf5272.h"
#include "init.h"
#include "can.h"
#include "canvar.h"

void ext_irq1_handler(void); 

/********************************************************************/

__interrupt__
void ext_irq1_handler (void) 
{

/*This is the interrupt service routine called when data is received*/

 MCF5272_IMM *imm = mcf5272_get_immp();

 printf("ext_irq1_handler\n"); 

/*Read received data and output over M5272C3 terminal uart*/
 CAN_MsgObj_RxData(Msg1,8);

/*Set Int1 IPL =6, for /Out1*/	
 MCF5272_WR_SIM_ICR1(imm,0xE8888888);				

/*Msg obj interrupt pending flag reset*/
 CAN_MsgObj_IntReset(Msg1);
 
/*Reset transmission message object, inhibit transmission*/
 CAN_MsgObjTx_Reset(Msg0); 	

/*Alternate message object 0 Tx data*/
if (toggle)
  {
   CAN_MsgObj_TxData(Msg0, 8, 0x55AA55AA, 0x11001100);
   toggle = 0;
  } 
 else
  {
   CAN_MsgObj_TxData(Msg0, 8, 0x11001100, 0x55AA55AA);
   toggle = 1; 
  }   

/*Set message object to start transmission*/
CAN_MsgObjTx_Start(Msg0);
}


void main ()
{

mcf5272_wr_sr (MCF5200_SR_IPL_0);

/*Initialise the QSPI module*/
mcf5272_qspi_init();

/*CAN node A and B initialisation*/
CAN_Node_Init();

/*Assign Msg0 to CAN node A, to transmit 8 bytes of data,  with standard ID of 2*/
CAN_MsgObj_Init(A, Msg0, Tx, 8, Stand, 2);

/*Assign Msg1 to CAN node B, to receive 8 bytes of data, with standard ID of 2*/
CAN_MsgObj_Init(B, Msg1, Rx, 8, Stand, 2);

/*Load Msg0 transmit data*/
CAN_MsgObj_TxData(Msg0, 8, 0xAA55AA55, 0x55AA55AA);

/*Enable Msg1 receive interrupt, assign to interrupt node 1 and /OUT1*/
CAN_MsgObj_IntEnable(Msg1,Rx,1);

/*Enable Msg1 to receive*/
CAN_MsgObjRx_Enable(Msg1);

/*Enable Msg0 to begin transmitting*/
CAN_MsgObjTx_Start(Msg0);

	while (1)
   	   ;	
}

/********************************************************************/
