/*********************************************************************
 *
 * Copyright:
 *	1999-2000 MOTOROLA, INC. All Rights Reserved.  
 *  You are hereby granted a copyright license to use, modify, and
 *  distribute the SOFTWARE so long as this entire notice is
 *  retained without alteration in any modified and/or redistributed
 *  versions, and that such modified versions are clearly identified
 *  as such. No licenses are granted by implication, estoppel or
 *  otherwise under any patents or trademarks of Motorola, Inc. This 
 *  software is provided on an "AS IS" basis and without warranty.
 *
 *  To the maximum extent permitted by applicable law, MOTOROLA 
 *  DISCLAIMS ALL WARRANTIES WHETHER EXPRESS OR IMPLIED, INCLUDING 
 *  IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR
 *  PURPOSE AND ANY WARRANTY AGAINST INFRINGEMENT WITH REGARD TO THE 
 *  SOFTWARE (INCLUDING ANY MODIFIED VERSIONS THEREOF) AND ANY 
 *  ACCOMPANYING WRITTEN MATERIALS.
 * 
 *  To the maximum extent permitted by applicable law, IN NO EVENT
 *  SHALL MOTOROLA BE LIABLE FOR ANY DAMAGES WHATSOEVER (INCLUDING 
 *  WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS 
 *  INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY
 *  LOSS) ARISING OF THE USE OR INABILITY TO USE THE SOFTWARE.   
 * 
 *  Motorola assumes no responsibility for the maintenance and support
 *  of this software
 ********************************************************************/

/*
 * File:		usb.c
 * Purpose:		Device Driver for the USB module of the MCF5272
 */

#include "mcf5272.h"
#include "init.h"

/********************************************************************/

/* Global Endpoint Status Structures */
USB_EP_STATE ep[NUM_ENDPOINTS];

/* Global Remote Wakeup Flag */
volatile int fRemoteWakeup = 0;

/* Global USB Descriptor Data (application specific) */
extern USB_DEVICE_DESC Descriptors;

/********************************************************************/
void 
usb_init(void)
{
	uint32 i, DescSize;
	uint32 *pConfigRam;
	uint32 *pDevDesc;
	MCF5272_IMM *imm = mcf5272_get_immp();

	/* Initialize Descriptor pointers and variables */
	pConfigRam = (uint32 *)((uint32)imm + MCF5272_USB_CFG_RAM);
	pDevDesc = (uint32 *)usb_get_desc(-1, -1, -1, -1);
	DescSize = usb_get_desc_size() + 3;

	/* Initialize Endpoint status structures */
	ep[0].ttype = CONTROL;
	ep[0].packet_size = ((USB_DEVICE_DESC *)pDevDesc)->bMaxPacketSize0;
	ep[0].fifo_length = (uint16)(ep[0].packet_size * FIFO_DEPTH);
	ep[0].buffer.start = 0;
	ep[0].buffer.free = 0;
	for (i = 1; i < NUM_ENDPOINTS; i++)
		ep[i].ttype = DISABLED;

	/* Invalidate Configuration RAM and disable Endpoint 0 */
	MCF5272_WR_USB_EP0CTL(imm, 0);

	/* Load the Configuration RAM with the descriptors */
	for (i = 0; i < (DescSize/4); i++)
		pConfigRam[i] = pDevDesc[i];

	/* Initialize FIFOs */
	usb_fifo_init();

	/* Initialize the Interrupts */
	usb_isr_init();

	/* Enable USB controller, Config RAM, EXT AFE */
#if (DEBUG)
	MCF5272_WR_USB_EP0CTL(imm, 0
		| MCF5272_USB_EP0CTL_DEBUG
//		| MCF5272_USB_EP0CTL_AFE_EN
		| MCF5272_USB_EP0CTL_USB_EN
		| MCF5272_USB_EP0CTL_CFG_RAM_VAL);
#else
	MCF5272_WR_USB_EP0CTL(imm, 0
//		| MCF5272_USB_EP0CTL_AFE_EN
		| MCF5272_USB_EP0CTL_USB_EN
		| MCF5272_USB_EP0CTL_CFG_RAM_VAL);
#endif
}

/********************************************************************/
void 
usb_isr_init(void)
{
	MCF5272_IMM *imm = mcf5272_get_immp();

	/* Clear any pending interrupts in all Endpoints */
	MCF5272_WR_USB_EP0ISR(imm, 0x0001FFFF);
	MCF5272_WR_USB_EP1ISR(imm, 0x001F);
	MCF5272_WR_USB_EP2ISR(imm, 0x001F);
	MCF5272_WR_USB_EP3ISR(imm, 0x001F);
	MCF5272_WR_USB_EP4ISR(imm, 0x001F);
	MCF5272_WR_USB_EP5ISR(imm, 0x001F);
	MCF5272_WR_USB_EP6ISR(imm, 0x001F);
	MCF5272_WR_USB_EP7ISR(imm, 0x001F);

	/* Enable all interrupts on all Endpoints */
	/* These will be altered on configuration/interface changes */
	MCF5272_WR_USB_EP0IMR(imm, 0x0001FFFF);
	MCF5272_WR_USB_EP1IMR(imm, 0x001F);
	MCF5272_WR_USB_EP2IMR(imm, 0x001F);
	MCF5272_WR_USB_EP3IMR(imm, 0x001F);
	MCF5272_WR_USB_EP4IMR(imm, 0x001F);
	MCF5272_WR_USB_EP5IMR(imm, 0x001F);
	MCF5272_WR_USB_EP6IMR(imm, 0x001F);
	MCF5272_WR_USB_EP7IMR(imm, 0x001F);

	/* Initialize Interrupt Control Registers */
	MCF5272_WR_SIM_ICR2(imm, 0
		| (0x00008888)
		| (USB_EP0_LEVEL << 12) 
		| (USB_EP1_LEVEL << 8) 
		| (USB_EP2_LEVEL << 4) 
		| (USB_EP3_LEVEL << 0));
	MCF5272_WR_SIM_ICR3(imm, 0
		| (0x88880000)
		| (USB_EP4_LEVEL << 28) 
		| (USB_EP5_LEVEL << 24) 
		| (USB_EP6_LEVEL << 20) 
		| (USB_EP7_LEVEL << 16));
}

/********************************************************************/
void 
usb_endpoint0_isr(void)
{
	MCF5272_IMM *imm = mcf5272_get_immp();
	uint32 event;

	event = MCF5272_RD_USB_EP0ISR(imm) & MCF5272_RD_USB_EP0IMR(imm);

	if (event & MCF5272_USB_EP0ISR_DEV_CFG)
	{
		usb_devcfg_service();
		/* Clear this interrupt bit */
		MCF5272_WR_USB_EP0ISR(imm, MCF5272_USB_EP0ISR_DEV_CFG);
	}
	if (event & MCF5272_USB_EP0ISR_VEND_REQ)
	{
		if ((MCF5272_RD_USB_DRR1(imm) & IN) && 
			(MCF5272_RD_USB_DRR2(imm) & 0xFFFF0000))
		{
			/* Set the IN-BUSY bit */
			MCF5272_WR_USB_EP0CTL(imm,
				  MCF5272_RD_USB_EP0CTL(imm) 
				| MCF5272_USB_EP0CTL_IN_BUSY);
		}
		/* Is this a GET_DESCRIPTOR(String) request? */
		if ((MCF5272_RD_USB_DRR1(imm) & 0xFF00 >> 8) == GET_DESCRIPTOR)
		{
			/* To do: create a Tx routine */
			printf("Host requested the String Descriptor\n");
			while (1) ;
		}
		/* Clear this interrupt bit */
		MCF5272_WR_USB_EP0ISR(imm, MCF5272_USB_EP0ISR_VEND_REQ);
		usb_vendreq_service(
			(uint8)(MCF5272_RD_USB_DRR1(imm) & 0xFF),
			(uint8)(MCF5272_RD_USB_DRR1(imm) >> 8),
			(uint16)(MCF5272_RD_USB_DRR1(imm) >> 16),
			(uint16)(MCF5272_RD_USB_DRR2(imm) & 0xFFFF),
			(uint16)(MCF5272_RD_USB_DRR2(imm) >> 16));
	}
	if (event & MCF5272_USB_EP0ISR_FRM_MAT)
	{
		/* Clear this interrupt bit */
		MCF5272_WR_USB_EP0ISR(imm, MCF5272_USB_EP0ISR_FRM_MAT);
	}
	if (event & MCF5272_USB_EP0ISR_ASOF)
	{
		/* Clear this interrupt bit */
		MCF5272_WR_USB_EP0ISR(imm, MCF5272_USB_EP0ISR_ASOF);
	}
	if (event & MCF5272_USB_EP0ISR_SOF)
	{
		/* Clear this interrupt bit */
		MCF5272_WR_USB_EP0ISR(imm, MCF5272_USB_EP0ISR_SOF);
	}
	if (event & MCF5272_USB_EP0ISR_WAKE_CHG)
	{
        if (MCF5272_RD_USB_EP0SR(imm) & MCF5272_USB_EP0SR_WAKE_ST)
        {
            /* Enable Wake-on-Ring */
            MCF5272_WR_USB_EP0CTL(imm,
                  MCF5272_RD_USB_EP0CTL(imm)
                | MCF5272_USB_EP0CTL_WOR_EN);
            /* Wake-on-Ring function is invoked when /INT1 pin is 0 */
            MCF5272_WR_USB_EP0CTL(imm,
                  MCF5272_RD_USB_EP0CTL(imm)
                & ~MCF5272_USB_EP0CTL_WOR_LVL);
 		    #if (DEBUG)
			    printf("Wake-on-Ring enabled.\n");
		    #endif
        }
        else
        {
			/* Disable Wake-on-Ring */
			MCF5272_WR_USB_EP0CTL(imm,
				   MCF5272_RD_USB_EP0CTL(imm)
				& ~MCF5272_USB_EP0CTL_WOR_EN);
			#if (DEBUG)
				printf("Wake-on-Ring disabled.\n");
			#endif
        }
        /* Clear this interrupt bit */
		MCF5272_WR_USB_EP0ISR(imm, MCF5272_USB_EP0ISR_WAKE_CHG);
	}
	if (event & MCF5272_USB_EP0ISR_RESUME)
	{
        usb_bus_state_chg_service(RESUME);
		/* Clear this interrupt bit */
		MCF5272_WR_USB_EP0ISR(imm, MCF5272_USB_EP0ISR_RESUME);
	}
	if (event & MCF5272_USB_EP0ISR_SUSPEND)
	{
        if (MCF5272_RD_USB_EP0SR(imm) & MCF5272_USB_EP0SR_WAKE_ST)
            usb_bus_state_chg_service(SUSPEND | ENABLE_WAKEUP);
        else
            usb_bus_state_chg_service(SUSPEND);
		/* Clear this interrupt bit */
		MCF5272_WR_USB_EP0ISR(imm, MCF5272_USB_EP0ISR_SUSPEND);
	}
	if (event & MCF5272_USB_EP0ISR_RESET)
	{
        usb_bus_state_chg_service(RESET);
		/* Clear this interrupt bit */
		MCF5272_WR_USB_EP0ISR(imm, MCF5272_USB_EP0ISR_RESET);
	}
	if (event & ( MCF5272_USB_EP0ISR_OUT_EOT
				| MCF5272_USB_EP0ISR_OUT_EOP
				| MCF5272_USB_EP0ISR_OUT_LVL))
	{
		usb_out_service(0, event);
	}
	if (event & ( MCF5272_USB_EP0ISR_IN_EOT
				| MCF5272_USB_EP0ISR_IN_EOP
				| MCF5272_USB_EP0ISR_IN_LVL))
	{
		usb_in_service(0, event);
	}
	if (event & MCF5272_USB_EP0ISR_UNHALT)
	{
		#if (DEBUG)
			printf("Endpoint 0 has been UNHALTed\n");
		#endif
		/* Clear this interrupt bit */
		MCF5272_WR_USB_EP0ISR(imm, MCF5272_USB_EP0ISR_UNHALT);
	}
	if (event & MCF5272_USB_EP0ISR_HALT)
	{
		#if (DEBUG)
			printf("Endpoint 0 has been HALTed\n");
		#endif
		/* Clear this interrupt bit */
		MCF5272_WR_USB_EP0ISR(imm, MCF5272_USB_EP0ISR_HALT);
	}
}

/********************************************************************/
void 
usb_endpoint_isr(uint32 epnum)
{
	int event;
	MCF5272_IMM *imm = mcf5272_get_immp();

	#if (DEBUG)
		printf("Received interrupt for Endpoint %d\n",epnum);
	#endif

	event = MCF5272_RD_USB_EPISR(imm, epnum) 
		  & MCF5272_RD_USB_EPIMR(imm, epnum);

	if (event & ( MCF5272_USB_EPNISR_EOT	
				| MCF5272_USB_EPNISR_EOP
				| MCF5272_USB_EPNISR_FIFO_LVL))
	{
		/* IN Endpoint */
		if (MCF5272_RD_USB_EPISR(imm, epnum) & MCF5272_USB_EPNISR_DIR)
			usb_in_service(epnum, event);
		/* OUT Endpoint */
		else
			usb_out_service(epnum,event);
	}
	if (event & MCF5272_USB_EPNISR_HALT)
	{
		/* Call Project specific routine */
		usb_ep_halt(epnum);
		MCF5272_WR_USB_EPISR(imm, epnum, MCF5272_USB_EPNISR_HALT);
	}
	if (event & MCF5272_USB_EPNISR_UNHALT)
	{
		/* Call Project specific routine */
		usb_ep_unhalt(epnum);
		MCF5272_WR_USB_EPISR(imm, epnum, MCF5272_USB_EPNISR_UNHALT);
	}

}

/********************************************************************/
void 
usb_in_service(uint32 epnum, uint32 event)
{
	MCF5272_IMM *imm = mcf5272_get_immp();
	uint16 i;
	uint32 free_space;

	#if (DEBUG)
		if (event & MCF5272_USB_EP0ISR_IN_EOT)
			printf("Received IN_EOT for EP #%d\n",epnum);
		if (event & MCF5272_USB_EP0ISR_IN_EOP)
			printf("Received IN_EOP for EP #%d\n",epnum);
		if (event & MCF5272_USB_EP0ISR_IN_LVL)
			printf("Received IN_LVL for EP #%d\n",epnum);
	#endif

	/* Clear the FIFO Level interrupt now */
	if (event & MCF5272_USB_EPNISR_FIFO_LVL)
		MCF5272_WR_USB_EPISR(imm, epnum, MCF5272_USB_EPNISR_FIFO_LVL);

	if (event & (MCF5272_USB_EPNISR_EOP | MCF5272_USB_EPNISR_FIFO_LVL))
	{	
		/* Clear this interrupt bit */
		MCF5272_WR_USB_EPISR(imm, epnum, MCF5272_USB_EPNISR_EOP);

		if (ep[epnum].buffer.position < ep[epnum].buffer.length)
		{
			/* USBEPDP0 only monitors the OUT FIFO */
			if (epnum == 0)
				free_space = ep[0].packet_size;
			else
				free_space = (uint16)(ep[epnum].fifo_length 
								- MCF5272_RD_USB_EPDPR(imm, epnum));
	
			for (i = ep[epnum].buffer.position; \
				 i < ep[epnum].buffer.position + free_space; \
				 i++)
			{
				/* Write from the buffer to the FIFO */
				MCF5272_WR_USB_EPDR(imm, epnum, 8, ep[epnum].buffer.start[i]);
				if (i == ep[epnum].buffer.length)
				{
					/* Clear the IN_BUSY bit */
					MCF5272_WR_USB_EPCTL(imm, epnum,
						  MCF5272_RD_USB_EPCTL(imm, epnum)
						& ~MCF5272_USB_EPNCTL_IN_BUSY);
					break;
				}
			}
			ep[epnum].buffer.position = i;
		}
	}
	if ((event & MCF5272_USB_EPNISR_FIFO_LVL) && (ep[epnum].ttype == ISOCHRONOUS))
	{
		/* Clear this interrupt bit */
		MCF5272_WR_USB_EPISR(imm, epnum, 0
			| MCF5272_USB_EPNISR_FIFO_LVL );
		if (ep[epnum].buffer.start && 
			 (ep[epnum].buffer.length == ep[epnum].buffer.position))
		{
			usb_ep_tx_done(epnum);
			ep[epnum].buffer.start = 0;
			ep[epnum].buffer.length = 0;
			ep[epnum].buffer.position = 0;
			ep[epnum].buffer.free = 0;

		}
	}
	if ((event & MCF5272_USB_EPNISR_EOT) ||
		 (ep[epnum].buffer.position == ep[epnum].buffer.length))
	{
		MCF5272_WR_USB_EPISR(imm, epnum, 0
			| MCF5272_USB_EPNISR_EOT);
		if (ep[epnum].buffer.start)
		{
			if (ep[epnum].buffer.free)
				free(ep[epnum].buffer.start);
			/* Call the Tx Handler */
			usb_ep_tx_done(epnum);
		}
		ep[epnum].buffer.start = 0;
		ep[epnum].buffer.length = 0;
		ep[epnum].buffer.position = 0;
		ep[epnum].buffer.free = 0;
	}
}

/********************************************************************/
void 
usb_out_service(uint32 epnum, uint32 event)
{
	MCF5272_IMM *imm = mcf5272_get_immp();
	uint32 i;
	uint16 fifo_data;

	#if (DEBUG)
		if (event & (MCF5272_USB_EP0ISR_OUT_EOT | MCF5272_USB_EPNISR_EOT))
			printf("Received OUT_EOT on EP%d\n",epnum);
		if (event & (MCF5272_USB_EP0ISR_OUT_EOP | MCF5272_USB_EPNISR_EOP))
			printf("Received OUT_EOP on EP%d\n",epnum);
		if (event & (MCF5272_USB_EP0ISR_OUT_LVL | MCF5272_USB_EPNISR_FIFO_LVL))
			printf("Received OUT_LVL on EP%d\n",epnum);
	#endif
	
	if (event & (MCF5272_USB_EP0ISR_OUT_LVL | MCF5272_USB_EPNISR_FIFO_LVL))
	{
		/* Clear the FIFO Level Interrupt bits now */
		MCF5272_WR_USB_EPISR(imm, epnum, 0
			| MCF5272_USB_EP0ISR_OUT_LVL
			| MCF5272_USB_EPNISR_FIFO_LVL);
	}
	if (event & ( MCF5272_USB_EP0ISR_OUT_LVL 
				| MCF5272_USB_EPNISR_FIFO_LVL
				| MCF5272_USB_EP0ISR_OUT_EOP
				| MCF5272_USB_EPNISR_EOP))
	{
		/* Clear the EOP interrupt bits */
		MCF5272_WR_USB_EPISR(imm, epnum, 0 
			| MCF5272_USB_EP0ISR_OUT_EOP
			| MCF5272_USB_EPNISR_EOP);

		/* Read the Data Present register */
		fifo_data = MCF5272_RD_USB_EPDPR(imm, epnum);

		if (ep[epnum].buffer.start &&
		   (ep[epnum].buffer.position + fifo_data > ep[epnum].buffer.length))
		{
			/* Buffer is going to overflow! */
			/* To do: realloc() to avoid this problem */
			usb_ep_rx_done(epnum,OVERFLOW_ERROR);
			ep[epnum].buffer.start = 0;
		}
		if (ep[epnum].buffer.start == 0)
		{
			/* No Buffer allocated! */
			ep[epnum].buffer.start = (uint8 *) malloc(BUFFER_SIZE);
			if (!ep[epnum].buffer.start)
			{
				/* Could not allocate buffer */
				usb_ep_rx_done(epnum,MALLOC_ERROR);
				return;
			}
			ep[epnum].buffer.free = TRUE;
			ep[epnum].buffer.length = BUFFER_SIZE;
			ep[epnum].buffer.position = 0;
		}

		/* Read the data from the FIFO into the buffer */
		for (i = ep[epnum].buffer.position; \
			 i < ep[epnum].buffer.position + fifo_data;)
		{
			if (ep[epnum].buffer.position + fifo_data - i > 3)
			{
				*((uint32 *)(&ep[epnum].buffer.start[i])) = 
					MCF5272_RD_USB_EPDR(imm, epnum,32);
				i += 4;
			}
			else
			{
				ep[epnum].buffer.start[i] = 
					MCF5272_RD_USB_EPDR(imm, epnum,8);
				i++;
			}
		}
		ep[epnum].buffer.position += fifo_data;
	}
	if (event & (MCF5272_USB_EP0ISR_OUT_LVL | MCF5272_USB_EPNISR_FIFO_LVL))
	{
		/* Clear the FIFO Level Interrupt bits */
		MCF5272_WR_USB_EPISR(imm, epnum, 0
			| MCF5272_USB_EP0ISR_OUT_LVL
			| MCF5272_USB_EPNISR_FIFO_LVL);
	}
	if (event & (MCF5272_USB_EP0ISR_OUT_EOT | MCF5272_USB_EPNISR_EOT))
	{
		/* The current transfer is complete */
		/* Clear the EOT Interrupt bits */
		MCF5272_WR_USB_EPISR(imm, epnum, 0
			| MCF5272_USB_EP0ISR_OUT_EOT
			| MCF5272_USB_EPNISR_EOT);

		/* Call the Rx Handler */
		usb_ep_rx_done(epnum,SUCCESS);

		/* Free the buffer if it was malloc()ed */
		if (ep[epnum].buffer.free)
		{
			ep[epnum].buffer.free = 0;
			free(ep[epnum].buffer.start);
		}
		ep[epnum].buffer.length = 0;
		ep[epnum].buffer.position = 0;
		ep[epnum].buffer.start = 0;
	}
}

/********************************************************************/
void 
usb_bus_state_chg_service(uint32 event)
{
    if (event & SUSPEND)
    {
    	#if (DEBUG)
			printf("The bus is suspended (J state)\n");
		#endif
        if (event & ENABLE_WAKEUP)
        {
            #if (DEBUG)
                printf("Remote wakeup enabled.\n");
            #endif
            /* Set the Remote Wakeup flag */
            fRemoteWakeup = 1;
        }
    }
    if (event & RESUME)
    {
		#if (DEBUG)
			printf("The bus has been resumed (K state)\n");
		#endif
    }
    if (event & RESET)
    {
		#if (DEBUG)
			printf("USB reset signal detected\n");
		#endif
    }
}

/********************************************************************/
void 
usb_devcfg_service(void)
{
	static uint8 last_addr = 0;
	static uint8 last_config = 0;
	static uint32 last_altsetting = 0;
	uint8 flags, i, j, epnum;
	USB_CONFIG_DESC *pCfgDesc;
	USB_INTERFACE_DESC *pIfcDesc;
	USB_ENDPOINT_DESC *pEpDesc;
	MCF5272_IMM *imm = mcf5272_get_immp();

	flags = 0;
	if (last_addr != MCF5272_RD_USB_FAR(imm))
	{
		#if (DEBUG)
			printf("Address change:  old addr = %d, new addr = %d\n",
					last_addr, MCF5272_RD_USB_FAR(imm));
		#endif
		last_addr = MCF5272_RD_USB_FAR(imm);
		flags |= ADDRESS_CHG;
	}
	if (last_config != (MCF5272_RD_USB_EP0SR(imm) >> 12))
	{
		#if (DEBUG)
			printf("Configuration change:  old cfg = %d, new cfg = %d\n",
					last_config, MCF5272_RD_USB_EP0SR(imm) >> 12);
		#endif

		last_config = (uint8)(MCF5272_RD_USB_EP0SR(imm) >> 12);
		flags |= CONFIGURATION_CHG;
	}
	if (last_altsetting != (MCF5272_RD_USB_ASR(imm)))
	{
		#if (DEBUG)
			printf("Interface change:  old setting = %d, new setting = %d\n",
					last_altsetting, MCF5272_RD_USB_ASR(imm));
		#endif
		last_altsetting = (MCF5272_RD_USB_ASR(imm));
		flags |= INTERFACE_CHG;
	}
	if (flags & (INTERFACE_CHG | CONFIGURATION_CHG))
	{
		/* Initialize Endpoint settings for this configuration */

		/* Start by marking all Endpoints as disabled (except EP0) */
		for (i = 1; i < NUM_ENDPOINTS; i++)
			ep[i].ttype = DISABLED;
		/* Get pointer to active Configuration descriptor */
		pCfgDesc = (USB_CONFIG_DESC *)usb_get_desc(last_config, -1, -1, -1);
		/* Iterate through all the valid Interfaces in this Configuration */
		for (i = 0; i < pCfgDesc->bNumInterfaces; i++)
		{
			pIfcDesc = (USB_INTERFACE_DESC *)usb_get_desc(last_config, i, 
														AltSetting(imm, i),
														-1);
			/* Iterate through all the valid Endpoints in this interface */
			for (j = 0; j < pIfcDesc->bNumEndpoints; j++)
			{
				pEpDesc = (USB_ENDPOINT_DESC *)usb_get_desc(last_config, i, 
														  AltSetting(imm, i),
														  j);
				if (pEpDesc)
				{
					epnum = pEpDesc->bEndpointAddress & 0x0F;
					/* See if this Endpoint is actually active */
					if (!(MCF5272_RD_USB_EPISR(imm, epnum) & 
						  MCF5272_USB_EPNISR_PRES))
						return;
					/* Set the new transfer type */
					ep[epnum].ttype = pEpDesc->bmAttributes;
					/* Set the current direction */
					if (MCF5272_RD_USB_EPISR(imm, epnum) & 
						MCF5272_USB_EPNISR_DIR)
						ep[epnum].dir = IN;
					else
						ep[epnum].dir = OUT;
					/* Get the current packet size */
					ep[epnum].packet_size = (uint16)(pEpDesc->wMaxPacketSizeL + 
						(pEpDesc->wMaxPacketSizeH << 8));
					/* No buffer allocated yet */
					ep[epnum].buffer.start = 0;
					ep[epnum].buffer.free = 0;

					if (ep[epnum].ttype == ISOCHRONOUS)
					{
						/* Set the FIFO length */
						ep[epnum].fifo_length = ep[epnum].packet_size;
						/* Enable ISO mode and set FIFO Level for interrupts */
						MCF5272_WR_USB_EPCTL(imm, epnum, 0
							| MCF5272_USB_EPNCTL_ISO_MODE
							| MCF5272_USB_EPNCTL_FIFO_LVL(2));
						/* Disable EOP and EOT interrupts, enable FIFO_LVL */
						MCF5272_WR_USB_EPIMR(imm, epnum, 
							(MCF5272_RD_USB_EPIMR(imm, epnum)
								& ~(MCF5272_USB_EPNISR_EOP
								| MCF5272_USB_EPNISR_EOT))
							| MCF5272_USB_EPNISR_FIFO_LVL);
					}
					else
					{
						/* Set the FIFO length */
						ep[epnum].fifo_length = \
							(uint16)(ep[epnum].packet_size * FIFO_DEPTH);
						/* Disable ISO mode */
						MCF5272_WR_USB_EPCTL(imm, epnum, 0);
						/* Enable EOP and EOT interrupts, disable FIFO_LVL */
						MCF5272_WR_USB_EPIMR(imm, epnum,
							(MCF5272_RD_USB_EPIMR(imm, epnum)
								& ~MCF5272_USB_EPNISR_FIFO_LVL)
							| MCF5272_USB_EPNISR_EOP
							| MCF5272_USB_EPNISR_EOT);
					}
				}
			}
		}	
		usb_fifo_init();
	}
	
	/* Call application specific routine to notify of config/iface change */
	usb_devcfg_notice(flags, last_config, last_altsetting);
}

/********************************************************************/
void
usb_vendreq_done(uint32 error)
{
	MCF5272_IMM *imm = mcf5272_get_immp();

	if (error)
		/* This sends a single STALL as the handshake */
		MCF5272_WR_USB_EP0CTL(imm,
			MCF5272_RD_USB_EP0CTL(imm)
			| MCF5272_USB_EP0CTL_CMD_OVER
			| MCF5272_USB_EP0CTL_CMD_ERR);
	else
		MCF5272_WR_USB_EP0CTL(imm,
			MCF5272_RD_USB_EP0CTL(imm)
			| MCF5272_USB_EP0CTL_CMD_OVER);
	return;
}

/********************************************************************/
void
usb_sort_ep_array(USB_EP_STATE *list[], int n)
{
	/* Simple bubble sort function called by usb_fifo_init() */
	int i, pass, sorted;
	USB_EP_STATE *temp;
	pass = 1;

	do {
		sorted = 1;
		for (i = 0; i < n - pass; ++i)
		{
			if (list[i]->fifo_length < list[i+1]->fifo_length )
			{
				/* Exchange out-of-order pair */
				temp = list[i];
				list[i] = list[i + 1];
				list[i+1] = temp;
				sorted = 0;
			}
		}
		++pass;
	} while(!sorted);
}

/********************************************************************/
void
usb_make_power_of_two(uint32 *size)
{
	/* Called by usb_fifo_init() */
	int i, not_power_of_two = 0;
	uint32 new_size = *size;

	for (i = 0; new_size != 1; i++)
	{
		if (new_size & 0x0001)
			not_power_of_two = 1;
		new_size >>= 1;
	}
	new_size = 1 << (i + not_power_of_two);
	if (new_size > 256)
		new_size = 256;
	*size = new_size;
}

/********************************************************************/
void
usb_fifo_init(void)
{
	MCF5272_IMM *imm = mcf5272_get_immp();
	USB_EP_STATE *pIN[NUM_ENDPOINTS];
	USB_EP_STATE *pOUT[NUM_ENDPOINTS];
	uint8 nIN, nOUT, i;
	uint16 INpos, OUTpos;

	/* Endpoint 0 is always present and bi-directional */
	pIN[0] = &ep[0];
	pOUT[0] = &ep[0];
	nIN = nOUT = 1;

	/* Sort the active endpoints by direction */
	for (i = 1; i < NUM_ENDPOINTS; i++)
	{
		if (ep[i].ttype != DISABLED)
		{
			if (ep[i].dir == IN)
				pIN[nIN++] = &ep[i];
			else
				pOUT[nOUT++] = &ep[i];
		}
	}

	/* Make sure FIFO size is a power of 2; if not, make it so */
	for (i = 0; i < nIN; i++)
		usb_make_power_of_two( &(pIN[i]->fifo_length) );
	for (i = 0; i < nOUT; i++)
		usb_make_power_of_two( &(pOUT[i]->fifo_length) );

	/* Sort the endpoints by FIFO length (decending) */
	usb_sort_ep_array(pIN, nIN);
	usb_sort_ep_array(pOUT, nOUT);

	/* Calculate the FIFO address for each endpoint */
	INpos = 0;
	OUTpos = 0;
	for (i = 0; i < nIN; i++)
	{
		pIN[i]->in_fifo_start = INpos;
		INpos += pIN[i]->fifo_length;
	}
	for (i = 0; i < nOUT; i++)
	{
		pOUT[i]->out_fifo_start = OUTpos;
		OUTpos += pOUT[i]->fifo_length;
	}

	/* Save the FIFO settings into the Configuration Registers */
	
	/* Initialize Endpoint 0 IN FIFO */
	MCF5272_WR_USB_IEP0CFG(imm, 0
		| (ep[0].packet_size << 22)
		| (ep[0].fifo_length << 11)
		|  ep[0].in_fifo_start);

	/* Initialize Endpoint 0 OUT FIFO */
	MCF5272_WR_USB_OEP0CFG(imm, 0
		| (ep[0].packet_size << 22)
		| (ep[0].fifo_length << 11)
		|  ep[0].out_fifo_start);

	for (i = 1; i < NUM_ENDPOINTS; i++)
	{
		if (ep[i].ttype != DISABLED)
		{
			if (ep[i].dir == IN)
				/* Initialize Endpoint i FIFO */
				MCF5272_WR_USB_EPCFG(imm, i, 0
				| (ep[i].packet_size << 22)
				| (ep[i].fifo_length << 11)
				|  ep[i].in_fifo_start);
			else
				/* Initialize Endpoint i FIFO */
				MCF5272_WR_USB_EPCFG(imm, i, 0
				| (ep[i].packet_size << 22)
				| (ep[i].fifo_length << 11)
				|  ep[i].out_fifo_start);
		}
	}
}

/********************************************************************/
uint32
usb_tx_data(uint32 epnum, uint8 *start, uint16 length)
{
	MCF5272_IMM *imm = mcf5272_get_immp();
	uint32 imr_mask, i;

	/* Make sure there is data to send */
	if (!start)
		return 0;
	/* Check the bounds on epnum */
	if (epnum > NUM_ENDPOINTS)
		return 0;
	/* See if the EP is currently busy */
	if (ep[epnum].buffer.start || (epnum && MCF5272_RD_USB_EPDPR(imm, epnum)))
		return 0;
	/* Make sure this is an IN endpoint */
	if (epnum && !(MCF5272_RD_USB_EPISR(imm, epnum) & MCF5272_USB_EPNISR_DIR))
		return 0;
	/* Make sure this EP is not HALTed */
	if (MCF5272_RD_USB_EPISR(imm, epnum) & MCF5272_USB_EPNISR_HALT)
		return 0;

	/* Setup the EP Buffer structure */
	ep[epnum].buffer.start = start;
	ep[epnum].buffer.length = length;
	ep[epnum].buffer.position = 0;
	
	/* Set the IN-BUSY bit */
	MCF5272_WR_USB_EPCTL(imm, epnum,
		  MCF5272_RD_USB_EPCTL(imm, epnum) 
		| MCF5272_USB_EPNCTL_IN_BUSY);

	/* Save off the current IMR */
	imr_mask = MCF5272_RD_USB_EPIMR(imm, epnum);

	/* Clear the EOT, EOP and FIFO_LVL mask bits */
	MCF5272_WR_USB_EPIMR(imm, epnum, imr_mask & 
		~(MCF5272_USB_EPNISR_EOT
		| MCF5272_USB_EPNISR_EOP
		| MCF5272_USB_EPNISR_FIFO_LVL));

	for (i = 0; i < ep[epnum].fifo_length; )
	{
		if ( (ep[epnum].buffer.length - i) <= 3 )
		{
			MCF5272_WR_USB_EPDR(imm, epnum, 8, ep[epnum].buffer.start[i]);
			i++;
		}
		else
		{
			MCF5272_WR_USB_EPDR(imm, epnum, 32,
				*(uint32 *)(&ep[epnum].buffer.start[i]));
			i += 4;
		}
		if ( i == ep[epnum].buffer.length )
		{
			/* This is all of the data to be sent -> Clear the IN_BUSY bit */
			MCF5272_WR_USB_EPCTL(imm, epnum, MCF5272_RD_USB_EPCTL(imm, epnum)
				& ~MCF5272_USB_EPNCTL_IN_BUSY);
			break;
		}
	}
	ep[epnum].buffer.position = (uint16)i;
	MCF5272_WR_USB_EPIMR(imm, epnum, imr_mask);
	return 1;
}
/********************************************************************/
uint8*
usb_get_desc(int8 config, int8 iface, int8 setting, int8 ep)
{
	/* Note:
	 * ep is the offset and not the physical endpoint number
	 * In order to get the config desc pointer, specify -1 for 
	 * all other inputs except config
	 * In order to get the interface/alternate setting desc pointer,
	 * specify -1 for epNum
	 */
	int i;
	uint8 *pDesc = (uint8 *)&Descriptors;

	if (config != -1)
	{
		if (config > ((USB_DEVICE_DESC *)pDesc)->bNumConfigurations)
			return 0;
		
		/* Allow for non-standard desc between device and config desc */
		while (pDesc[1] != CONFIGURATION)
			pDesc += pDesc[0];	

		/* pDesc now points to Config 1 descriptor */
		for (i = 1; i < config;)
		{
			pDesc += pDesc[0];
			if (pDesc[1] == CONFIGURATION)
				i++;
		}

		/* pDesc now points to the correct Configuration descriptor */
		if ((iface != -1) && (setting != -1))
		{
			if (iface >= ((USB_CONFIG_DESC *)pDesc)->bNumInterfaces)
				return 0;

			/* Allow for non-standard desc between config and iface desc */
			while (pDesc[1] != INTERFACE)
				pDesc += pDesc[0];

			/* pDesc now points to first Interface descriptor */
			for (i = 0; i < iface;)
			{
				pDesc += pDesc[0];
				if (pDesc[1] == INTERFACE && pDesc[3] == 0)
					i++;
			}
			/* pDesc now points to correct Interface descriptor */
			for (i = 0; i < setting;)
			{
				pDesc += pDesc[0];
				if (pDesc[1] == INTERFACE)
				{
					if (pDesc[2] != iface)
						return 0;
					else
						i++;
				}
			}

			/* pDesc now points to correct Alternate Setting descriptor */
			if (ep != -1)
			{
				if (ep >= pDesc[4])
					return 0;

				/* Allow for non-standard desc between iface and ep desc */
				while (pDesc[1] != ENDPOINT)
					pDesc += pDesc[0];

				/* pDesc now points to first Endpoint descriptor */
				for (i = 0; i < ep;)
				{
					pDesc += pDesc[0];
					if (pDesc[1] == ENDPOINT)
						i++;
				}
				/* pDesc now points to the correct Endpoint descriptor */
			}
		}
	}
	return (pDesc);
}

/********************************************************************/
